# **************************************************************************
# *
# * Authors:     J.M. De la Rosa Trevin (jmdelarosa@cnb.csic.es)
# *
# * Unidad de  Bioinformatica of Centro Nacional de Biotecnologia , CSIC
# *
# * This program is free software; you can redistribute it and/or modify
# * it under the terms of the GNU General Public License as published by
# * the Free Software Foundation; either version 3 of the License, or
# * (at your option) any later version.
# *
# * This program is distributed in the hope that it will be useful,
# * but WITHOUT ANY WARRANTY; without even the implied warranty of
# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# * GNU General Public License for more details.
# *
# * You should have received a copy of the GNU General Public License
# * along with this program; if not, write to the Free Software
# * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
# * 02111-1307  USA
# *
# *  All comments concerning this program package may be sent to the
# *  e-mail address 'scipion@cnb.csic.es'
# *
# **************************************************************************
"""
Configuration definition for launching the ShowJ visualization utility.
The name of ShowJ is due historical reasons. This data visualization tools
was first called xmipp_show. And later was re-written using Java and 
became xmipp_showj.
"""
import logging
logger = logging.getLogger(__name__)

import os
import sys
from os.path import join
from collections import OrderedDict
import subprocess

from pyworkflow import Config
from pyworkflow.utils import getFreePort, runCommand
from pwem import Config as emConfig
import threading
import shlex
import socketserver
from pyworkflow.constants import SCIPION_DOMAIN, SCIPION_HOME_VAR, SCIPION_DEBUG

javaVersion = None

# ----------------------- Showj constants ---------------------------
COL_RENDER_NONE = 0
COL_RENDER_ID = 1
COL_RENDER_TEXT = 2
COL_RENDER_IMAGE = 3
COL_RENDER_CHECKBOX = 4
COL_RENDER_VOLUME = 5

PATH = 'path'
DATASET = 'dataset'

TABLE_NAME = 'blockComboBox'
COLS_CONFIG = 'tableLayoutConfiguration'
COLS_CONFIG_DEFAULT = 'defaultColumnsLayoutProperties'
LABEL_SELECTED = 'labelsToRenderComboBox'

MODE = 'mode'
MODE_GALLERY = 'gallery'
MODE_TABLE = 'table'
MODE_MD = 'metadata'
MODE_VOL_ASTEX = 'volume_astex'
MODE_VOL_CHIMERA = 'volume_chimera'
MODE_VOL_JSMOL = 'volume_jsmol'
RENDER = 'render'
ORDER = 'order'
VISIBLE = 'visible'
ZOOM = 'zoom'
SORT_BY = 'sortby'
LABELS = 'labels'
CLASSIFIER = 'classifier'

SAMPLINGRATE = 'sampling_rate'
CHIMERA_PORT = 'chimera_port'
INVERTY = 'inverty'

OBJCMDS = 'object_commands'

GOTO = 'goto'
ROWS = 'rows'
COLS = 'cols'
ALLOW_RENDER = 'allowRender'
MANUAL_ADJUST = 'colRowMode'

SELECTEDITEMS = 'listSelectedItems'
ENABLEDITEMS = 'listEnabledItems'
CHANGES = 'listChangesItems'
OLDMODE = 'oldModeShowj'
RENDERITEMS = 'renderItems'

VOL_SELECTED = 'volumesToRenderComboBox'
VOL_TYPE = 'typeVolume'
VOL_VIEW = 'resliceComboBox'

IMG_DIMS = 'imageDimensions'
IMG_ZOOM = 'zoom'
IMG_ZOOM_DEFAULT = 'defaultZoom'
IMG_MIRRORY = 'mirrorY'
IMG_APPLY_TRANSFORM = 'applyTransformMatrix'
IMG_ONLY_SHIFTS = 'onlyShifts'
IMG_WRAP = 'wrap'
IMG_MAX_WIDTH = 'imageMaxWidth'
IMG_MIN_WIDTH = 'imageMinWidth'
IMG_MAX_HEIGHT = 'imageMaxHeight'
IMG_MIN_HEIGHT = 'imageMinHeight'

PROJECT_NAME = 'projectName'
PROJECT_PATH = 'projectPath'
OBJECT_ID = 'objectId'


def getJvmMaxMemory():
    # Memory in GB
    return int(emConfig.JAVA_MAX_MEMORY)


class ColumnsConfig:
    """ Store the configuration of the columns for a given table in a dataset.
    The order of the columns will be stored and configuration for each columns.
    For each column, we store properties:
    - visible
    - allowSetVisible
    - renderable
    - allowSetRenderable
    - editable
    - allowSetEditable
    - renderFunc
    - renderFuncExtra
    """
    def __init__(self, table, allowRender=True, defaultColumnsLayout={}):

        self._columnsDict = OrderedDict()

        for col in table.iterColumns():
            colDefaultLayout = defaultColumnsLayout.get(col.getLabel(), {})
            col_properties = ColumnProperties(col, allowRender, colDefaultLayout)
            self._columnsDict[col.getName()] = col_properties

    def getRenderableColumns(self, extra=None):
        """ Return a list with the name of renderable columns.
            extra: parameter used to keep some rendering columns."""
        columnsName = []
        columnsLabel = []

        for col in self._columnsDict.values():
            if col.isRenderable():
                if extra is not None:
                    if col.getName() in extra:
                        columnsName.append(col.getName())
                        columnsLabel.append(col.getLabel())

                else:
                    columnsName.append(col.getName())
                    columnsLabel.append(col.getLabel())
#       columns = [col.getName() for col in self._columnsDict.values() if col.isRenderable()]
        return columnsName, columnsLabel

    def hasEnableColumn(self):
        for columnLayout in self._columnsDict.values():
            if "enable" == columnLayout.getLabel():
                return True
        return False

    def getColumnProperty(self, colName, propName):
        """ Get some property value of a given column. """
        col = self._columnsDict[colName]
        return getattr(col, propName)

    def configColumn(self, colName, **kwargs):
        """ Configure properties of a given column. """
        col = self._columnsDict[colName]
        for k, v in kwargs.items():
            setattr(col, k, v)

    def printColumns(self):
        for col in self._columnsDict.values():
            print("column: ", col.getLabel())
            print("  values: ", col.getValues())


class ColumnProperties:
    """ Store some properties to customize how each column
    will be display in the table.
    """
    def __init__(self, col, allowRender, defaultColumnLayoutProperties):
        self._column = col
        self.columnType = col.getRenderType()

        if 'visible' in defaultColumnLayoutProperties:
            self.visible = defaultColumnLayoutProperties['visible']
        else:
            self.visible = not (self.columnType == COL_RENDER_ID)

        self.allowSetVisible = True

        self.editable = (self.columnType == COL_RENDER_TEXT)
        self.allowSetEditable = self.editable

        self.renderable = ('renderable' in defaultColumnLayoutProperties
                           and defaultColumnLayoutProperties['renderable'].lower() == 'true')

        self.allowSetRenderable = ((self.columnType == COL_RENDER_IMAGE or
                                   self.columnType == COL_RENDER_VOLUME) and allowRender)

        self.renderFunc = "get_image"
        self.extraRenderFunc = ""

    def getLabel(self):
        return self._column.getLabel()

    def getName(self):
        return self._column.getName()

    def getColumnType(self):
        return self.columnType

    def allowsRenderable(self):
        return self.renderable or self.allowSetRenderable

    def isRenderable(self):
        return self.renderable or self.allowSetRenderable

    def setValues(self, defaultColumnLayoutProperties):
        for key in defaultColumnLayoutProperties:
            setattr(self, key, defaultColumnLayoutProperties[key])

    def getValues(self):
        return {"visible": self.visible,
                "allowSetVisible": self.allowSetVisible,
                "editable": self.editable,
                "allowSetEditable": self.allowSetEditable,
                "renderable": self.renderable,
                "allowSetRenderable": self.allowSetRenderable,
                "renderFunc": self.renderFunc,
                "extraRenderFunc": self.extraRenderFunc,
                'columnType': self.columnType,
                'columnName': self.getName(),
                'columnLabel': self.getLabel()
                }


def getArchitecture():
    import platform
    arch = platform.architecture()[0]
    for a in ['32', '64']:
        if a in arch:
            return a
    return 'NO_ARCH'

def getJavaVersion():
    global javaVersion

    if javaVersion is None:
        version = subprocess.check_output([getJavaBin(), '-version'], stderr=subprocess.STDOUT)
        javaVersion = int(version.decode("utf-8").split('"')[1].split('.')[0])

    return javaVersion
def getJavaBin():

    if emConfig.JAVA_HOME:
        return join(emConfig.JAVA_HOME, "bin", "java")
    else:
        return  "java"

def getJavaIJappArguments(memory, appName, appArgs):
    """ Build the command line arguments to launch
    a Java application based on ImageJ.
    memory: the amount of memory passed to the JVM.
    appName: the qualified name of the java application.
    appArgs: the arguments specific to the application.
    """
    if memory is None:
        memory = getJvmMaxMemory()

    jdkLib = join(emConfig.JAVA_HOME, 'lib')
    javaBind = join(emConfig.XMIPP_HOME, "bindings", "java")
    imagej_home = join(javaBind, "imagej")
    lib = join(emConfig.XMIPP_HOME, "lib")
    javaLib = join(javaBind, 'lib')
    plugins_dir = os.path.join(imagej_home, "plugins")
    arch = getArchitecture()

    if getJavaVersion() > 9:
        args = "-Xmx%(memory)sg -Djava.library.path=%(lib)s -Dplugins.dir=%(plugins_dir)s -cp %(jdkLib)s/*:%(imagej_home)s/*:%(javaLib)s/* %(appName)s %(appArgs)s" % locals()
    else:
        args = "-Xmx%(memory)sg -d%(arch)s -Djava.library.path=%(lib)s -Dplugins.dir=%(plugins_dir)s -cp %(jdkLib)s/*:%(imagej_home)s/*:%(javaLib)s/* %(appName)s %(appArgs)s" % locals()

    return args


def runJavaIJapp(memory, appName, args, env=None):
    env = env or {}
    getEnviron = Config.getDomain().importFromPlugin('xmipp3', 'Plugin').getEnviron
    env.update(getEnviron(xmippFirst=False))

    # Add scipion variables
    env[SCIPION_DOMAIN] = Config.SCIPION_DOMAIN
    env[SCIPION_HOME_VAR] = Config.SCIPION_HOME

    if Config.debugOn():
        env[SCIPION_DEBUG] = "1"
    
    # Add out python path to the PATH
    env["PATH"] = ":".join([sys.executable, env.get("PATH", "")])
    # Try chimera
    try:
        chimeraPlugin = Config.getDomain().importFromPlugin('chimera', 'Plugin', doRaise=True)
        env["CHIMERA_LAUNCHER"] = chimeraPlugin.getProgram() + " %s"
    except Exception as e:
        pass

    args = getJavaIJappArguments(memory, appName, args)
    javaBin = getJavaBin()
    logger.info('%s %s' % (javaBin, args))
    cmd = [javaBin] + shlex.split(args)

    return subprocess.Popen(cmd, env=env)


def launchSupervisedPickerGUI(micsFn, outputDir, protocol,
                              mode=None, memory=None,
                              pickerProps=None, inTmpFolder=False):
    app = "xmipp.viewer.particlepicker.training.SupervisedPickerRunner"
    args = "--input %s --output %s" % (micsFn, outputDir)

    if mode:
        args += " --mode %s" % mode

    if pickerProps:
        args += " --classifier " + pickerProps
    else:
        port = initProtocolTCPServer(protocol)
        args += " --scipion %s" % port

    if inTmpFolder:
        args += " --tmp true"

    return runJavaIJapp(memory, app, args)


def launchTiltPairPickerGUI(micsFn, outputDir, protocol, mode=None):
    port = initProtocolTCPServer(protocol)
    app = "xmipp.viewer.particlepicker.tiltpair.TiltPairPickerRunner"
    args = "--input %s --output %s  --scipion %s" % (micsFn, outputDir, port)
    if mode:
        args += " --mode %s" % mode
    return runJavaIJapp(None, app, args)


class ProtocolTCPRequestHandler(socketserver.BaseRequestHandler):

    def handle(self):  # FIXME: RUNNING FOREVER
        protocol = self.server.protocol
        msg = str(self.request.recv(1024), "utf-8")
        tokens = shlex.split(msg)
        if msg.startswith('run function'):
            functionName = tokens[2]
            # try:
            functionPointer = getattr(protocol, functionName)
            functionPointer(*tokens[3:])
            self.request.sendall(b'done\n')
            self.server.end = True
            # except:
            # print 'protocol %s must implement %s'%(protocol.getName(), functionName)

        else:
            answer = b'no answer available\n'
            self.request.sendall(answer)


class MySocketServer(socketserver.TCPServer):

    def serve_forever(self):
        self.end = False
        while not self.end:
            self.handle_request()


def initProtocolTCPServer(protocol):
    address = ''
    port = getFreePort()
    server = MySocketServer((address, port), ProtocolTCPRequestHandler)
    server.protocol = protocol
    server_thread = threading.Thread(name="ProtocolTCPServer",
                                     target=server.serve_forever)
    server_thread.daemon = True
    server_thread.start()
    return port



# **************************************************************************
# *
# * Authors:     J.M. De la Rosa Trevin (delarosatrevin@scilifelab.se) [1]
# *
# * [1] SciLifeLab, Stockholm University
# *
# * This program is free software; you can redistribute it and/or modify
# * it under the terms of the GNU General Public License as published by
# * the Free Software Foundation; either version 3 of the License, or
# * (at your option) any later version.
# *
# * This program is distributed in the hope that it will be useful,
# * but WITHOUT ANY WARRANTY; without even the implied warranty of
# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# * GNU General Public License for more details.
# *
# * You should have received a copy of the GNU General Public License
# * along with this program; if not, write to the Free Software
# * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
# * 02111-1307  USA
# *
# *  All comments concerning this program package may be sent to the
# *  e-mail address 'scipion@cnb.csic.es'
# *
# **************************************************************************
import enum
import os
from datetime import datetime
from collections import OrderedDict

import pyworkflow.object as pwobj
import pyworkflow.protocol.params as params
import pyworkflow.protocol as pwprot
import pyworkflow.utils as pwutils

import pwem.objects as emobj
from pwem.protocols import ProtParticles


class ProtParticlePickingOutput(enum.Enum):
    """ Possible outputs for particle picking protocols
    """
    outputCoordinates = emobj.SetOfCoordinates


class ProtParticlePicking(ProtParticles):

    _possibleOutputs = ProtParticlePickingOutput
    OUTPUT_PREFIX = ProtParticlePickingOutput.outputCoordinates.name

    def _defineParams(self, form):

        form.addSection(label='Input')
        form.addParam('inputMicrographs', params.PointerParam,
                      pointerClass='SetOfMicrographs',
                      label=pwutils.Message.LABEL_INPUT_MIC, important=True,
                      help='Select the SetOfMicrographs to be used during '
                           'picking.')

    # -------------------------- INFO functions -------------------------------
    def getSummary(self, coordSet):
        summary = []
        summary.append("Number of particles picked: %s" % coordSet.getSize())
        summary.append("Particle size: %s" % coordSet.getBoxSize())
        return "\n".join(summary)

    def getMethods(self, output):
        msg = 'User picked %d particles ' % output.getSize()
        msg += 'with a particle size of %s.' % output.getBoxSize()
        return msg

    def _methods(self):
        methodsMsgs = []
        if self.getInputMicrographs() is None:
            return ['Input micrographs not available yet.']

        methodsMsgs.append("Input micrographs %s of size %d."
                           % (self.getObjectTag(self.getInputMicrographs()),
                              self.getInputMicrographs().getSize()))

        if self.getOutputsSize() >= 1:
            for key, output in self.iterOutputAttributes(emobj.SetOfCoordinates):
                msg = self.getMethods(output)
                methodsMsgs.append("%s: %s" % (self.getObjectTag(output), msg))
        else:
            methodsMsgs.append(pwutils.Message.TEXT_NO_OUTPUT_CO)

        return methodsMsgs

    def _summary(self):
        summary = []
        if self.getInputMicrographs() is not None:
            summary.append("Number of input micrographs: %d"
                           % self.getInputMicrographs().getSize())

        if self.getOutputsSize() >= 1:
            for key, output in self.iterOutputAttributes():
                summary.append("*%s:* \n %s " % (key, output.getObjComment()))
        else:
            summary.append(pwutils.Message.TEXT_NO_OUTPUT_CO)
        return summary

    def getInputMicrographsPointer(self):
        return self.inputMicrographs

    def getInputMicrographs(self):
        return self.getInputMicrographsPointer().get()

    def _getCoords(self, CoordClass):
        result = None
        for _, attr in self.iterOutputAttributes(CoordClass):
            result = attr  # Get the last output that is SetOfCoordinates or so
        return result

    def getCoords(self):
        return self._getCoords(emobj.SetOfCoordinates)

    def getCoordsTiltPair(self):
        from pwem.objects.data_tiltpairs import CoordinatesTiltPair
        return self._getCoords(CoordinatesTiltPair)

    def _createOutput(self, outputDir):
        micSet = self.getInputMicrographs()
        suffix = self.__getOutputSuffix()
        outputName = self.OUTPUT_PREFIX + suffix
        coordSet = self._createSetOfCoordinates(micSet, suffix)
        self.readSetOfCoordinates(outputDir, coordSet)
        coordSet.setObjComment(self.getSummary(coordSet))
        outputs = {outputName: coordSet}
        self._defineOutputs(**outputs)
        self._defineSourceRelation(self.getInputMicrographsPointer(), coordSet)

    def createOutputStep(self):
        self._createOutput(self._getExtraPath())

    def readSetOfCoordinates(self, workingDir, coordSet):
        pass

    def getCoordsDir(self):
        pass

    def __getOutputSuffix(self):
        """ Get the name to be used for a new output.
        For example: outputCoordinates7.
        It should take into account previous outputs
        and number with a higher value.
        """
        maxCounter = -1
        for attrName, _ in self.iterOutputAttributes(emobj.SetOfCoordinates):
            suffix = attrName.replace(self.OUTPUT_PREFIX, '')
            try:
                counter = int(suffix)
            except Exception as ex:
                counter = 1  # when there is not number assume 1
            maxCounter = max(counter, maxCounter)

        return str(maxCounter+1) if maxCounter > 0 else ''  # empty if not output

    def registerCoords(self, coordsDir):
        """ This method is usually inherited by all Pickers
        and it is used from the Java picking GUI to register
        a new SetOfCoordinates when the user click on +Particles button. 
        """
        from pwem import Domain
        suffix = self.__getOutputSuffix()
        outputName = self.OUTPUT_PREFIX + suffix

        readSetOfCoordinates = Domain.importFromPlugin('xmipp3.convert',
                                                       'readSetOfCoordinates')
        inputset = self.getInputMicrographs()
        # micrographs are the input set if protocol is not finished
        outputset = self._createSetOfCoordinates(inputset, suffix=suffix)
        readSetOfCoordinates(coordsDir, outputset.getMicrographs(), outputset)
        summary = self.getSummary(outputset)
        outputset.setObjComment(summary)
        outputs = {outputName: outputset}
        self._defineOutputs(**outputs)

        # Using a pointer to define the relations is more robust to scheduling
        # and id changes between the protocol run.db and the main project
        # database. The pointer defined below points to the outputset object
        self._defineSourceRelation(self.getInputMicrographsPointer(),
                                   pwobj.Pointer(value=self,
                                                 extended=outputName))
        self._store()


class ProtParticlePickingAuto(ProtParticlePicking):
    """ A derived class from ProtParticlePicking to differentiate those
    picking protocols that works in automatic way, i.e., that are not
    interactive and are also good candidates to be run in streaming. """

    def _insertAllSteps(self):
        self.initialIds = self._insertInitialSteps()
        self.micDict = OrderedDict()
        pwutils.makeFilePath(self._getAllDone())

        micDict, self.streamClosed = self._loadInputList()
        pickMicIds = self._insertNewMicsSteps(micDict.values())

        self._insertFinalSteps(pickMicIds)

    def _insertInitialSteps(self):
        """ Override this function to insert some steps before the
        picking micrograph steps.
        Should return a list of ids of the initial steps. """
        return []

    def _insertFinalSteps(self, micSteps):
        """ Override this function to insert some steps after the
        picking micrograph steps.
        Receive the list of step ids of the picking steps. """
        self._insertFunctionStep(self.createOutputStep.__name__,
                                 prerequisites=micSteps, wait=True)

    def _getPickArgs(self):
        """ Should be implemented in sub-classes to define the argument
        list that should be passed to the picking step function.
        """
        return []

    def _insertPickMicrographStep(self, mic, prerequisites, *args):
        """ Basic method to insert a picking step for a given micrograph. """
        micStepId = self._insertFunctionStep('pickMicrographStep',
                                             mic.getMicName(), *args,
                                             prerequisites=prerequisites)
        return micStepId

    def pickMicrographStep(self, micName, *args):
        """ Step function that will be common for all picking protocols.
        It will take care of re-building the micrograph object from the micDict
        argument and perform any conversion if needed. Then, the function
        _pickMicrograph will be called, that should be implemented by each
        picking protocol.
        """
        mic = self.micDict[micName]
        micDoneFn = self._getMicDone(mic)
        micFn = mic.getFileName()

        if self.isContinued() and os.path.exists(micDoneFn):
            self.info("Skipping micrograph: %s, seems to be done" % micFn)
            return

        # Clean old finished files
        pwutils.cleanPath(micDoneFn)

        self.info("Picking micrograph: %s " % micFn)
        self._pickMicrograph(mic, *args)

        # Mark this mic as finished
        open(micDoneFn, 'w').close()

    def _pickMicrograph(self, mic, *args):
        """ This function should be implemented by subclasses in order
        to picking the given micrograph. """
        pass

    # Group of functions to pick several micrographs if the batch size is
    # defined. In some programs it might be more efficient to pick many
    # at once and not one by one

    def _insertPickMicrographListStep(self, micList, prerequisites, *args):
        """ Basic method to insert a picking step for a given micrograph. """
        micNameList = [mic.getMicName() for mic in micList]
        micStepId = self._insertFunctionStep('pickMicrographListStep',
                                             micNameList, *args,
                                             prerequisites=prerequisites)
        return micStepId

    def pickMicrographListStep(self, micNameList, *args):
        micList = []

        for micName in micNameList:
            mic = self.micDict[micName]
            micDoneFn = self._getMicDone(mic)
            micFn = mic.getFileName()
            if self.isContinued() and os.path.exists(micDoneFn):
                self.info("Skipping micrograph: %s, seems to be done" % micFn)

            else:
                # Clean old finished files
                pwutils.cleanPath(micDoneFn)
                self.info("Picking micrograph: %s " % micFn)
                micList.append(mic)

        self._pickMicrographList(micList, *args)

        for mic in micList:
            # Mark this mic as finished
            open(self._getMicDone(mic), 'w').close()

    def _pickMicrographList(self, micList, *args):
        """ This function can be implemented by subclasses if it is a more
        efficient way to pick many micrographs at once.
         Default implementation will just call the _pickMicrograph
        """
        for mic in micList:
            self._pickMicrograph(mic, *args)

    # --------------------------- UTILS functions ----------------------------

    def _stepsCheck(self):
        # To allow streaming picking we need to detect:
        #   1) new micrographs ready to be picked
        #   2) new output coordinates that have been produced and add then
        #      to the output set.
        self._checkNewInput()
        self._checkNewOutput()

    def _insertNewMicsSteps(self, inputMics):
        """ Insert steps to process new mics (from streaming)
        Params:
            insertedDict: contains already processed mics
            inputMics: input mics set to be check
        """
        return self._insertNewMics(inputMics,
                                   lambda mic: mic.getMicName(),
                                   self._insertPickMicrographStep,
                                   self._insertPickMicrographListStep,
                                   *self._getPickArgs())

    def _loadSet(self, inputSet, SetClass, getKeyFunc):
        """ Load a given input set if their items are not already present
        in the self.micDict.
        This can be used to load new micrographs for picking as well as
        new CTF (if used) in streaming.
        """
        setFn = inputSet.getFileName()
        self.debug("Loading input db: %s" % setFn)
        updatedSet = SetClass(filename=setFn)
        updatedSet.loadAllProperties()
        newItemDict = OrderedDict()
        for item in updatedSet:
            micKey = getKeyFunc(item)
            if micKey not in self.micDict:
                newItemDict[micKey] = item.clone()
        streamClosed = updatedSet.isStreamClosed()
        updatedSet.close()
        self.debug("Closed db.")

        return newItemDict, streamClosed

    def _loadMics(self, micSet):
        return self._loadSet(micSet, emobj.SetOfMicrographs,
                             lambda mic: mic.getMicName())

    def _loadCTFs(self, ctfSet):
        return self._loadSet(ctfSet, emobj.SetOfCTF,
                             lambda ctf: ctf.getMicrograph().getMicName())

    def _loadInputList(self):
        """ Load the input set of micrographs that are ready to be picked. """
        return self._loadMics(self.getInputMicrographs())

    def _checkNewInput(self):
        # Check if there are new micrographs to process from the input set
        localFile = self.getInputMicrographs().getFileName()
        now = datetime.now()
        self.lastCheck = getattr(self, 'lastCheck', now)
        mTime = datetime.fromtimestamp(os.path.getmtime(localFile))
        self.debug('Last check: %s, modification: %s'
                   % (pwutils.prettyTime(self.lastCheck),
                      pwutils.prettyTime(mTime)))
        # If the input micrographs.sqlite have not changed since our last check,
        # it does not make sense to check for new input data
        if self.lastCheck > mTime and hasattr(self, 'listOfMics'):
            return None

        self.lastCheck = now
        # Open input micrographs.sqlite and close it as soon as possible
        micDict, self.streamClosed = self._loadInputList()
        newMics = micDict.values()
        outputStep = self._getFirstJoinStep()

        if newMics:
            fDeps = self._insertNewMicsSteps(newMics)
            if outputStep is not None:
                outputStep.addPrerequisites(*fDeps)
            self.updateSteps()

    def _checkNewOutput(self):
        if getattr(self, 'finished', False):
            return

        # Load previously done items (from text file)
        doneList = self._readDoneList()
        # Check for newly done items
        listOfMics = self.micDict.values()
        nMics = len(listOfMics)
        newDone = [m for m in listOfMics
                   if m.getObjId() not in doneList and self._isMicDone(m)]

        # Update the file with the newly done mics
        # or exit from the function if no new done mics
        self.debug('_checkNewOutput: ')
        self.debug('   listOfMics: %s, doneList: %s, newDone: %s'
                   % (nMics, len(doneList), len(newDone)))

        allDone = len(doneList) + len(newDone)
        # We have finished when there is not more input mics (stream closed)
        # and the number of processed mics is equal to the number of inputs
        self.finished = self.streamClosed and allDone == nMics
        streamMode = pwobj.Set.STREAM_CLOSED if self.finished else pwobj.Set.STREAM_OPEN
        self.debug('   streamMode: %s newDone: %s' % (streamMode,
                                                      not(newDone == [])))
        if newDone:
            newDoneUpdated = self._updateOutputCoordSet(newDone, streamMode)
            self._writeDoneList(newDoneUpdated)
        elif not self.finished:
            # If we are not finished and no new output have been produced
            # it does not make sense to proceed and updated the outputs
            # so we exit from the function here

            # Maybe it would be good idea to take a snap to avoid
            # so much IO if this protocol does not have much to do now
            if allDone == nMics:
                self._streamingSleepOnWait()

            return

        self.debug('   finished: %s ' % self.finished)
        self.debug('        self.streamClosed (%s) AND' % self.streamClosed)
        self.debug('        allDone (%s) == len(self.listOfMics (%s)'
                   % (allDone, nMics))

        if self.finished:  # Unlock createOutputStep if finished all jobs
            self._updateStreamState(streamMode)
            outputStep = self._getFirstJoinStep()
            if outputStep and outputStep.isWaiting():
                outputStep.setStatus(pwprot.STATUS_NEW)

    def _micIsReady(self, mic):
        """ Function to check if a micrograph (although reported done)
        is ready to update the coordinates from it. An practical use of this
        function will be for protocols that need to wait for the CTF of that
        micrograph to be ready as well.
        """
        return True

    def readCoordsFromMics(self, outputDir, micDoneList, outputCoords):
        """ This method should be implemented in subclasses to read
        the coordinates from a given list of micrographs.
        """
        pass

    def _updateOutputCoordSet(self, micList, streamMode):
        micDoneList = [mic for mic in micList if self._micIsReady(mic)]

        # Do no proceed if there is not micrograph ready
        if not micDoneList:
            return []

        outputName = 'outputCoordinates'
        outputDir = self.getCoordsDir()
        outputCoords = getattr(self, outputName, None)

        # If there are not outputCoordinates yet, it means that is the first
        # time we are updating output coordinates, so we need to first create
        # the output set
        firstTime = outputCoords is None

        if firstTime:
            micSetPtr = self.getInputMicrographsPointer()
            outputCoords = self._createSetOfCoordinates(micSetPtr)
        else:
            outputCoords.enableAppend()

        self.info("Reading coordinates from mics: %s" % ','.join([mic.strId() for mic in micList]))
        self.readCoordsFromMics(outputDir, micDoneList, outputCoords)
        self.debug(" _updateOutputCoordSet Stream Mode: %s " % streamMode)
        outputCoords.setObjComment(self.getSummary(outputCoords))
        self._updateOutputSet(outputName, outputCoords, streamMode)

        if firstTime:
            self._defineSourceRelation(self.getInputMicrographsPointer(),
                                       outputCoords)

        return micDoneList

    def _updateStreamState(self, streamMode):

        outputName = 'outputCoordinates'
        outputCoords = getattr(self, outputName, None)

        # If there are not outputCoordinates yet, it means that is the first
        # time we are updating output coordinates, so we need to first create
        # the output set
        firstTime = outputCoords is None

        if firstTime:
            micSetPtr = self.getInputMicrographsPointer()
            outputCoords = self._createSetOfCoordinates(micSetPtr)
        else:
            outputCoords.enableAppend()

        self.debug(" _updateStreamState Stream Mode: %s " % streamMode)
        self._updateOutputSet(outputName, outputCoords, streamMode)

    def _getMicDone(self, mic):
        return self._getExtraPath('DONE', 'mic_%06d.TXT' % mic.getObjId())

    def _isMicDone(self, mic):
        """ A mic is done if the marker file exists. """
        return os.path.exists(self._getMicDone(mic))

    def _getAllDone(self):
        return self._getExtraPath('DONE', 'all.TXT')

    def _readDoneList(self):
        """ Read from a text file the id's of the items that have been done. """
        doneFile = self._getAllDone()
        doneList = []
        # Check what items have been previously done
        if os.path.exists(doneFile):
            with open(doneFile) as f:
                doneList += [int(line.strip()) for line in f]

        return doneList

    def _writeDoneList(self, micList):
        """ Write to a text file the items that have been done. """
        doneFile = self._getAllDone()

        if not os.path.exists(doneFile):
            pwutils.makeFilePath(doneFile)

        with open(doneFile, 'a') as f:
            for mic in micList:
                f.write('%d\n' % mic.getObjId())

    def _getFirstJoinStepName(self):
        # This function will be used for streaming, to check which is
        # the first function that need to wait for all micrographs
        # to have completed, this can be overwritten in subclasses
        # (eg in Xmipp 'sortPSDStep')
        return self.createOutputStep.__name__

    def _getFirstJoinStep(self):
        for s in self._steps:
            if s.funcName == self._getFirstJoinStepName():
                return s
        return None

    def createOutputStep(self):
        # Not really required now
        # self._createOutput(self._getExtraPath())
        pass


class ProtPickingDifference(ProtParticlePicking):
    """
    Protocol to compute the difference between a reference SetOfParticles and
    a another set (usually a negative reference).

    The output will be a SetOfCoordinates with the particles in the reference
    input that are not close to coordinates in the negative input.

    """
    _label = 'picking difference'

    def _defineParams(self, form):
        form.addSection(label='Input')
        form.addParam('inputCoordinates', params.PointerParam,
                      pointerClass='SetOfCoordinates',
                      label="Input coordinates",
                      help='Select the reference set of coordinates.')
        form.addParam('negativeCoordinates', params.PointerParam,
                      pointerClass='SetOfCoordinates',
                      label="Negative coordinates",
                      help='Negative coordinates that will help to exclude '
                           'coordinates from the input set. ')
        form.addParam('differenceRadius', params.IntParam, default=10,
                      label="Radius (px)",
                      help="Distance radius to consider that two coordinates "
                           "close enough. If a particle in the input reference"
                           "set have any close particle in the negative set, "
                           "it will not be included in the output set. ")

        form.addParallelSection(threads=0, mpi=0)

    # ------------------------- INSERT steps functions -------------------------
    def _insertAllSteps(self):
        self._insertFunctionStep("createOutputStep",
                                 self.inputCoordinates.getObjId(),
                                 self.negativeCoordinates.getObjId(),
                                 self.differenceRadius.get())

    def getInputMicrographs(self):
        return self.inputCoordinates[0].get().getMicrographs()

    def _summary(self):
        summary = []
        return summary

    def _methods(self):
        return []

    def createOutputStep(self, inputId, negId, radius):
        inputCoords = self.inputCoordinates.get()
        negCoords = self.negativeCoordinates.get()
        inputMics = inputCoords.getMicrographs()
        outputCoords = self._createSetOfCoordinates(inputMics)
        outputCoords.setBoxSize(inputCoords.getBoxSize())

        r2 = radius * radius  # to avoid computing sqrt when comparing distances

        def _discard(coord, negPos):
            """ Find if there is a close negative particle to this one. """
            x, y = coord.getPosition()

            for nx, ny in negPos:
                if abs((x - nx) * (y - ny)) < r2:
                    return True

            return False  # Far enough from all negative coordinates

        # Read all consensus particles
        for mic in inputMics:
            negPos = [(c.getX(), c.getY())
                      for c in negCoords.iterCoordinates(mic)]
            for coord in inputCoords.iterCoordinates(mic):
                if not _discard(coord, negPos):
                    outputCoords.append(coord)

        # Set output
        self._defineOutputs(outputCoordinates=outputCoords)
        self._defineTransformRelation(self.inputCoordinates,
                                      self.outputCoordinates)
        self._defineSourceRelation(self.negativeCoordinates,
                                   self.outputCoordinates)



# -*- coding: utf-8 -*-
# **************************************************************************
# *
# * Authors:     Marta Martinez (mmmtnez@cnb.csic.es)
# *              Roberto Marabini (roberto@cnb.csic.es)
# *
# * Unidad de  Bioinformatica of Centro Nacional de Biotecnologia , CSIC
# *
# * This program is free software; you can redistribute it and/or modify
# * it under the terms of the GNU General Public License as published by
# * the Free Software Foundation; either version 3 of the License, or
# * (at your option) any later version.
# *
# * This program is distributed in the hope that it will be useful,
# * but WITHOUT ANY WARRANTY; without even the implied warranty of
# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# * GNU General Public License for more details.
# *
# * You should have received a copy of the GNU General Public License
# * along with this program; if not, write to the Free Software
# * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
# * 02111-1307  USA
# *
# *  All comments concerning this program package may be sent to the
# *  e-mail address 'scipion@cnb.csic.es'
# *
# **************************************************************************

import pyworkflow.tests as pwtests

import pwem.protocols as emprot
from pwem.objects.data import Alphabet

class TestImportBase(pwtests.BaseTest):
    @classmethod
    def setUpClass(cls):
        pwtests.setupTestProject(cls)
        cls.dsModBuild = pwtests.DataSet.getDataSet('model_building_tutorial')


class TestImportSequence(TestImportBase):
    USERID = "UserID"
    NAME = 'USER_SEQ'
    DESCRIPTION = 'User description'
    NUCLEOTIDESEQ1 = 'AATGCGGTTGGGBDSW********GGCACACG'
    AMINOACIDSSEQ1 = 'LARKJLAKPABXZJUO********VAVAVALK'
    CHAIN1 = '{"model": 0, "chain": "B", "residues": 148}'  # Protein
    CHAIN2 = '{"model": 0, "chain": "A", "residues": 12}'  # RNA
    CHAIN3 = '{"model": 0, "chain": "I", "residues": 146}'  # DNA
    pdbID1 = "3lqd"  # Protein
    pdbID2 = "205d"  # RNA
    pdbID3 = "1aoi"  # DNA and protein
    GENEBANKID = 'AJ520101.1'
    GENEBANKIDPROT = 'CAE8970392.1'
    UNIPROTID = 'P12345'

    def testImportUserNucleotideSequence1(self):
        """
        Import a single nucleotide sequence provided by the user (nucleotide
        alphabet by default)
        """
        args = {'inputSequenceName': self.NAME,
                'inputSequenceDescription': self.DESCRIPTION,
                'inputSequence': Alphabet.NUCLEOTIDES,
                'nucleotideIUPACalphabet': Alphabet.EXTENDED_DNA_ALPHABET,
                'inputRawSequence': self.NUCLEOTIDESEQ1
                }
        prot1 = self.newProtocol(emprot.ProtImportSequence, **args)
        prot1.setObjLabel('1_import DNA,\nseq from user\nExtended DNA '
                          'alphabet')
        self.launchProtocol(prot1)
        sequence = prot1.outputSequence
        self.assertEqual("USER_SEQ", sequence.getId())
        self.assertEqual("USER_SEQ", sequence.getSeqName())
        self.assertEqual("User description",
                         sequence.getDescription())
        self.assertEqual("AATGCGGTTG", sequence.getSequence()[:10])
        self.assertEqual(Alphabet.EXTENDED_DNA_ALPHABET, sequence.getAlphabet())

    def testImportUserNucleotideSequence2(self):
        """
        Import a single nucleotide sequence provided by the user (nucleotide
        alphabet AMBIGOUS_RNA_ALPHABET)
        """
        args = {'inputSequenceName': self.NAME,
                'inputSequenceDescription': self.DESCRIPTION,
                'inputSequence': Alphabet.NUCLEOTIDES,
                'nucleotideIUPACalphabet': Alphabet.AMBIGOUS_RNA_ALPHABET,
                'inputRawSequence': self.NUCLEOTIDESEQ1
                }
        prot2 = self.newProtocol(emprot.ProtImportSequence, **args)
        prot2.setObjLabel('2_import RNA,\nseq from user\nAmbiguous RNA '
                          'alphabet')
        self.launchProtocol(prot2)
        sequence = prot2.outputSequence
        self.assertEqual("USER_SEQ", sequence.getId())
        self.assertEqual("USER_SEQ", sequence.getSeqName())
        self.assertEqual("User description",
                         sequence.getDescription())
        self.assertEqual("AAGCGGGGGB", sequence.getSequence()[:10])
        self.assertEqual(Alphabet.AMBIGOUS_RNA_ALPHABET, sequence.getAlphabet())

    def testImportStructureNucleotideSequence1(self):
        """
        Import the sequence of chain A of atomic RNA structure 205d.cif
        """
        args = {'inputSequenceName': self.NAME,
                'inputSequenceDescription': self.DESCRIPTION,
                'inputSequence': Alphabet.NUCLEOTIDES,
                'inputNucleotideSequence':
                    emprot.ProtImportSequence.IMPORT_FROM_NUCLEOTIDE_STRUCTURE,
                'inputStructureSequence':
                    emprot.ProtImportSequence.IMPORT_STRUCTURE_FROM_ID,
                'pdbId': self.pdbID2,
                'inputStructureChain': self.CHAIN2
                }
        prot3 = self.newProtocol(emprot.ProtImportSequence, **args)
        prot3.setObjLabel('3_import RNA seq,\n from atomic '
                          'structure')
        self.launchProtocol(prot3)
        sequence = prot3.outputSequence
        self.assertEqual("205d__0_A", sequence.getId())
        self.assertEqual("USER_SEQ", sequence.getSeqName())
        self.assertEqual("User description",
                         sequence.getDescription())
        self.assertEqual("GGACUUUGGU", sequence.getSequence()[:10])
        self.assertEqual(Alphabet.UNAMBIGOUS_RNA_ALPHABET, sequence.getAlphabet()) # RNA

    def testImportStructureNucleotideSequence2(self):
        """
        Import the sequence of chain A of atomic RNA structure 205d.cif
        """
        args = {'inputSequenceID': self.USERID,
                'inputSequenceName': self.NAME,
                'inputSequenceDescription': self.DESCRIPTION,
                'inputSequence': Alphabet.NUCLEOTIDES,
                'inputNucleotideSequence':
                    emprot.ProtImportSequence.IMPORT_FROM_NUCLEOTIDE_STRUCTURE,
                'inputStructureSequence':
                    emprot.ProtImportSequence.IMPORT_STRUCTURE_FROM_FILES,
                'pdbFile': self.dsModBuild.getFile('PDBx_mmCIF/205d.cif'),
                'inputStructureChain': self.CHAIN2
                }
        prot4 = self.newProtocol(emprot.ProtImportSequence, **args)
        prot4.setObjLabel('4_import RNA seq,\n from atomic '
                          'structure')
        self.launchProtocol(prot4)
        sequence = prot4.outputSequence
        self.assertEqual("UserID", sequence.getId())
        self.assertEqual("USER_SEQ", sequence.getSeqName())
        self.assertEqual("User description",
                         sequence.getDescription())
        self.assertEqual("GGACUUUGGU", sequence.getSequence()[:10])
        self.assertEqual(Alphabet.UNAMBIGOUS_RNA_ALPHABET, sequence.getAlphabet()) # RNA

    def testImportStructureNucleotideSequence3(self):
        """
        Import the sequence of chain I of atomic DNA structure 1aoi.cif
        """
        args = {'inputSequenceName': self.NAME,
                'inputSequenceDescription': self.DESCRIPTION,
                'inputSequence': Alphabet.NUCLEOTIDES,
                'inputNucleotideSequence':
                    emprot.ProtImportSequence.IMPORT_FROM_NUCLEOTIDE_STRUCTURE,
                'inputStructureSequence':
                    emprot.ProtImportSequence.IMPORT_STRUCTURE_FROM_ID,
                'pdbId': self.pdbID3,
                'inputStructureChain': self.CHAIN3
                }
        prot5 = self.newProtocol(emprot.ProtImportSequence, **args)
        prot5.setObjLabel('5_import DNA seq,\n from atomic '
                          'structure')
        self.launchProtocol(prot5)
        sequence = prot5.outputSequence
        self.assertEqual("1aoi__0_I", sequence.getId())
        self.assertEqual("USER_SEQ", sequence.getSeqName())
        self.assertEqual("User description",
                         sequence.getDescription())
        self.assertEqual("ATCAATATCC", sequence.getSequence()[:10])
        self.assertEqual(Alphabet.UNAMBIGOUS_DNA_ALPHABET, sequence.getAlphabet()) # DNA

    def testImportStructureNucleotideSequence4(self):
        """
        Import the sequence of chain I of atomic DNA structure 1aoi.cif
        """
        args = {'inputSequenceID': self.USERID,
                'inputSequenceName': self.NAME,
                'inputSequenceDescription': self.DESCRIPTION,
                'inputSequence': Alphabet.NUCLEOTIDES,
                'inputNucleotideSequence':
                    emprot.ProtImportSequence.IMPORT_FROM_NUCLEOTIDE_STRUCTURE,
                'inputStructureSequence':
                    emprot.ProtImportSequence.IMPORT_STRUCTURE_FROM_FILES,
                'pdbFile': self.dsModBuild.getFile('PDBx_mmCIF/1aoi.cif'),
                'inputStructureChain': self.CHAIN3
                }
        prot6 = self.newProtocol(emprot.ProtImportSequence, **args)
        prot6.setObjLabel('6_import DNA seq,\n from atomic '
                          'structure')
        self.launchProtocol(prot6)
        sequence = prot6.outputSequence
        self.assertEqual("UserID", sequence.getId())
        self.assertEqual("USER_SEQ", sequence.getSeqName())
        self.assertEqual("User description",
                         sequence.getDescription())
        self.assertEqual("ATCAATATCC", sequence.getSequence()[:10])
        self.assertEqual(Alphabet.UNAMBIGOUS_DNA_ALPHABET, sequence.getAlphabet()) # DNA

    def testImportFileNucleotideSequence1(self):
        """
        Import a single nucleotide sequence from a text file
        """
        args = {'inputSequenceName': self.NAME,
                'inputSequence': Alphabet.NUCLEOTIDES,
                'inputNucleotideSequence':
                    emprot.ProtImportSequence.IMPORT_FROM_NUCLEOTIDE_FILES,
                'fileSequence': self.dsModBuild.getFile(
                    'Sequences/AJ520101.fasta')
                }
        prot7 = self.newProtocol(emprot.ProtImportSequence, **args)
        prot7.setObjLabel('7_import nucleotide,\nseq from file')
        self.launchProtocol(prot7)
        sequence = prot7.outputSequence
        self.assertEqual("AJ520101.1", sequence.getId())
        self.assertEqual('USER_SEQ', sequence.getSeqName())
        self.assertEqual("AJ520101.1 Rhizobium leguminosarum bv. viciae "
                         "plasmid "
                         "partial fixC gene, fixX gene, nifA gene "
                         "and nifB gene",
                         sequence.getDescription())
        self.assertEqual("GGATCCGAGA", sequence.getSequence()[:10])
        self.assertEqual(Alphabet.UNAMBIGOUS_DNA_ALPHABET, sequence.getAlphabet()) # DNA

    def testImportFileNucleotideSequence2(self):
        """
        Import a single nucleotide sequence from a text file
        """
        args = {'inputSequenceID': self.USERID,
                'inputSequenceName': self.NAME,
                'inputSequenceDescription': self.DESCRIPTION,
                'inputSequence': Alphabet.NUCLEOTIDES,
                'inputNucleotideSequence':
                    emprot.ProtImportSequence.IMPORT_FROM_NUCLEOTIDE_FILES,
                'fileSequence': self.dsModBuild.getFile(
                    'Sequences/AJ520101.fasta')
                }
        prot8 = self.newProtocol(emprot.ProtImportSequence, **args)
        prot8.setObjLabel('8_import nucleotide,\nseq from file')
        self.launchProtocol(prot8)
        sequence = prot8.outputSequence
        self.assertEqual("UserID", sequence.getId())
        self.assertEqual('USER_SEQ', sequence.getSeqName())
        self.assertEqual("User description",
                         sequence.getDescription())
        self.assertEqual("GGATCCGAGA", sequence.getSequence()[:10])
        self.assertEqual(Alphabet.UNAMBIGOUS_DNA_ALPHABET, sequence.getAlphabet()) # DNA

    def testImportFileNucleotideSequence3(self):
        """
        Import a single nucleotide sequence from a text file
        """
        args = {'inputSequenceName': self.NAME,
                'inputSequence': Alphabet.NUCLEOTIDES,
                'inputNucleotideSequence':
                    emprot.ProtImportSequence.IMPORT_FROM_NUCLEOTIDE_FILES,
                'fileSequence': self.dsModBuild.getFile(
                    'Sequences/Several_sequences.fasta')
                }
        prot9 = self.newProtocol(emprot.ProtImportSequence, **args)
        prot9.setObjLabel('9_import nucleotide,\nseq from file')
        self.launchProtocol(prot9)
        sequence = prot9.outputSequence
        self.assertEqual("Seq1", sequence.getId())
        self.assertEqual('USER_SEQ', sequence.getSeqName())
        self.assertEqual("Seq1 This is the sequence number one",
                         sequence.getDescription())
        self.assertEqual("TGGCTAAATA", sequence.getSequence()[:10])
        self.assertEqual(Alphabet.UNAMBIGOUS_DNA_ALPHABET, sequence.getAlphabet()) # DNA

    def testImportFileNucleotideSequence4(self):
        """
        Import a single nucleotide sequence from a text file that contains
        several sequences
        """
        args = {'inputSequenceID': self.USERID,
                'inputSequenceName': self.NAME,
                'inputSequenceDescription': self.DESCRIPTION,
                'inputSequence': Alphabet.NUCLEOTIDES,
                'inputNucleotideSequence':
                    emprot.ProtImportSequence.IMPORT_FROM_NUCLEOTIDE_FILES,
                'fileSequence': self.dsModBuild.getFile(
                    'Sequences/Several_sequences.fasta')
                }
        prot10 = self.newProtocol(emprot.ProtImportSequence, **args)
        prot10.setObjLabel('10_import nucleotide,\nseq from file')
        self.launchProtocol(prot10)
        sequence = prot10.outputSequence
        self.assertEqual("UserID", sequence.getId())
        self.assertEqual('USER_SEQ', sequence.getSeqName())
        self.assertEqual("User description",
                         sequence.getDescription())
        self.assertEqual("TGGCTAAATA", sequence.getSequence()[:10])
        self.assertEqual(Alphabet.UNAMBIGOUS_DNA_ALPHABET, sequence.getAlphabet()) # DNA

    def testImportGeneBankNucleotideSequence1(self):
        """
        Import a single nucleotide sequence from GeneBank
        """
        args = {'inputSequenceName': self.NAME,
                'inputSequence': Alphabet.NUCLEOTIDES,
                'inputNucleotideSequence':
                    emprot.ProtImportSequence.IMPORT_FROM_NUCLEOTIDE_GENEBANK,
                'geneBankSequence': self.GENEBANKID
                }
        prot11 = self.newProtocol(emprot.ProtImportSequence, **args)
        prot11.setObjLabel('11_import nucleotide,\nseq from '
                           'GeneBank')
        self.launchProtocol(prot11)
        sequence = prot11.outputSequence
        self.assertEqual("AJ520101.1", sequence.getId())
        self.assertEqual('USER_SEQ', sequence.getSeqName())
        self.assertEqual("AJ520101.1 Rhizobium leguminosarum bv. viciae "
                         "plasmid partial fixC gene, fixX gene, nifA gene "
                         "and nifB gene",
                         sequence.getDescription())
        self.assertEqual("GGATCCGAGA", sequence.getSequence()[:10])
        self.assertEqual(Alphabet.NUCLEOTIDES_ALPHABET, sequence.getAlphabet()) # DNA

    def testImportGeneBankNucleotideSequence2(self):
        """
        Import a single nucleotide sequence from GeneBank
        """
        args = {'inputSequenceName': self.NAME,
                'inputProteinSequence':
                    emprot.ProtImportSequence.IMPORT_FROM_PROTEIN_GENEBANK,
                'geneBankSequence': self.GENEBANKIDPROT
                }
        prot12 = self.newProtocol(emprot.ProtImportSequence, **args)
        prot12.setObjLabel('12.5_import protein,\nseq from '
                           'GeneBank')
        self.launchProtocol(prot12)
        sequence = prot12.outputSequence
        self.assertEqual(self.GENEBANKIDPROT, sequence.getId())
        self.assertEqual('USER_SEQ', sequence.getSeqName())
        self.assertEqual('CAE8970392.1 unnamed protein product, partial [Tetraselmis striata]',
                         sequence.getDescription())
        self.assertEqual("MPDHHLRYFR", sequence.getSequence()[:10])
        self.assertEqual(Alphabet.EXTENDED_PROTEIN_ALPHABET, sequence.getAlphabet()) # Protein

    def testImportUserAminoacidSequence1(self):
        """
        Import a single aminoacid sequence provided by the user (protein
        alphabet by default)
        """
        args = {'inputSequenceName': self.NAME,
                'inputSequenceDescription': self.DESCRIPTION,
                'inputRawSequence': self.AMINOACIDSSEQ1
                }
        prot13 = self.newProtocol(emprot.ProtImportSequence, **args)
        prot13.setObjLabel('13_import aminoacid seq,\n from user\nExtended '
                           'protein alphabet')
        self.launchProtocol(prot13)

        sequence = prot13.outputSequence
        self.assertEqual("USER_SEQ", sequence.getId())
        self.assertEqual("USER_SEQ", sequence.getSeqName())
        self.assertEqual("User description", sequence.getDescription())
        self.assertEqual("LARKJLAKPA", sequence.getSequence()[:10])
        self.assertEqual(Alphabet.EXTENDED_PROTEIN_ALPHABET, sequence.getAlphabet()) # Extended protein

    def testImportUserAminoacidSequence2(self):
        """
        Import a single aminoacid sequence provided by the user (protein
        alphabet PROTEIN_ALPHABET
        """
        args = {'inputSequenceName': self.NAME,
                'inputSequenceDescription': self.DESCRIPTION,
                'proteinIUPACalphabet': Alphabet.AMINOACIDS,
                'inputRawSequence': self.AMINOACIDSSEQ1
                }
        prot14 = self.newProtocol(emprot.ProtImportSequence, **args)
        prot14.setObjLabel('14_import aminoacid seq,\n from user\nProtein '
                           'alphabet')
        self.launchProtocol(prot14)
        sequence = prot14.outputSequence
        self.assertEqual("USER_SEQ", sequence.getId())
        self.assertEqual("USER_SEQ", sequence.getSeqName())
        self.assertEqual("User description", sequence.getDescription())
        self.assertEqual("LARKLAKPAV", sequence.getSequence()[:10])
        self.assertEqual(Alphabet.PROTEIN_ALPHABET, sequence.getAlphabet()) # Protein

    def testImportStructureAminoacidSequence1(self):
        """
        Import the sequence of chain B of atomic structure 3lqd.cif
        """
        args = {'inputSequenceName': self.NAME,
                'inputSequenceDescription': self.DESCRIPTION,
                'inputProteinSequence':
                    emprot.ProtImportSequence.IMPORT_FROM_STRUCTURE,
                'inputStructureSequence':
                    emprot.ProtImportSequence.IMPORT_STRUCTURE_FROM_ID,
                'pdbId': self.pdbID1,
                'inputStructureChain': self.CHAIN1
                }
        prot15 = self.newProtocol(emprot.ProtImportSequence, **args)
        prot15.setObjLabel('15_import aminoacid seq,\n from atomic '
                           'structure')
        self.launchProtocol(prot15)
        sequence = prot15.outputSequence

        self.assertEqual("3lqd__0_B", sequence.getId())
        self.assertEqual("USER_SEQ", sequence.getSeqName())
        self.assertEqual("User description", sequence.getDescription())
        self.assertEqual("VHLSGEEKSA", sequence.getSequence()[:10])
        self.assertEqual(Alphabet.EXTENDED_PROTEIN_ALPHABET, sequence.getAlphabet()) # Protein

    def testImportStructureAminoacidSequence2(self):
        """
        Import the sequence of chain B of atomic structure 3lqd.cif
        """
        args = {'inputSequenceID': self.USERID,
                'inputSequenceName': self.NAME,
                'inputSequenceDescription': self.DESCRIPTION,
                'inputProteinSequence':
                    emprot.ProtImportSequence.IMPORT_FROM_STRUCTURE,
                'inputStructureSequence':
                    emprot.ProtImportSequence.IMPORT_STRUCTURE_FROM_FILES,
                'pdbFile': self.dsModBuild.getFile('PDBx_mmCIF/3lqd.cif'),
                'inputStructureChain': self.CHAIN1
                }
        prot16 = self.newProtocol(emprot.ProtImportSequence, **args)
        prot16.setObjLabel('16_import aminoacid seq,\n from atomic '
                           'structure')
        self.launchProtocol(prot16)
        sequence = prot16.outputSequence
        self.assertEqual("UserID", sequence.getId())
        self.assertEqual("USER_SEQ", sequence.getSeqName())
        self.assertEqual("User description", sequence.getDescription())
        self.assertEqual("VHLSGEEKSA", sequence.getSequence()[:10])
        self.assertEqual(Alphabet.EXTENDED_PROTEIN_ALPHABET, sequence.getAlphabet()) # Protein

    def testImportFileAminoacidSequence1(self):
        """
        Import a single aminoacid sequence from a text file
        """
        args = {'inputSequenceName': self.NAME,
                'inputProteinSequence':
                    emprot.ProtImportSequence.IMPORT_FROM_FILES,
                'fileSequence': self.dsModBuild.getFile(
                    'Sequences/COX1_human.fasta')
                }
        prot17 = self.newProtocol(emprot.ProtImportSequence, **args)
        prot17.setObjLabel('17_import aminoacid,\nseq from file')
        self.launchProtocol(prot17)
        sequence = prot17.outputSequence

        self.assertEqual("YP_003024028.1", sequence.getId())
        self.assertEqual('USER_SEQ', sequence.getSeqName())
        self.assertEqual('YP_003024028.1 cytochrome c oxidase '
                         'subunit I (mitochondrion) [Homo sapiens]',
                         sequence.getDescription())
        self.assertEqual("MFADRWLFST", sequence.getSequence()[:10])
        self.assertEqual(Alphabet.EXTENDED_PROTEIN_ALPHABET, sequence.getAlphabet()) # Protein

    def testImportFileAminoacidSequence2(self):
        """
        Import a single aminoacid sequence from a text file
        """
        args = {'inputSequenceID': self.USERID,
                'inputSequenceName': self.NAME,
                'inputSequenceDescription': self.DESCRIPTION,
                'inputProteinSequence':
                    emprot.ProtImportSequence.IMPORT_FROM_FILES,
                'fileSequence': self.dsModBuild.getFile(
                    'Sequences/COX1_human.fasta')
                }
        prot18 = self.newProtocol(emprot.ProtImportSequence, **args)
        prot18.setObjLabel('18_import aminoacid,\nseq from file')
        self.launchProtocol(prot18)
        sequence = prot18.outputSequence
        self.assertEqual("UserID", sequence.getId())
        self.assertEqual('USER_SEQ', sequence.getSeqName())
        self.assertEqual('User description',
                         sequence.getDescription())
        self.assertEqual("MFADRWLFST", sequence.getSequence()[:10])
        self.assertEqual(Alphabet.EXTENDED_PROTEIN_ALPHABET, sequence.getAlphabet()) # Protein

    def testImportUniprotAminoacidSequence1(self):
        """
        Import a single aminoacid sequence from UniProt
        """
        args = {'inputSequenceName': self.NAME,
                'inputProteinSequence':
                    emprot.ProtImportSequence.IMPORT_FROM_UNIPROT,
                'uniProtSequence': self.UNIPROTID
                }
        prot19 = self.newProtocol(emprot.ProtImportSequence, **args)
        prot19.setObjLabel('19_import aminoacids,\nseq from '
                           'UniProt')
        self.launchProtocol(prot19)
        sequence = prot19.outputSequence
        self.assertEqual("P12345", sequence.getId())
        self.assertEqual('USER_SEQ', sequence.getSeqName())
        self.assertEqual('sp|P12345.2|AATM_RABIT RecName: Full=Aspartate aminotransferase, mitochondrial; Short=mAspAT; AltName: Full=Fatty acid-binding protein; Short=FABP-1; AltName: Full=Glutamate oxaloacetate transaminase 2; AltName: Full=Kynurenine aminotransferase 4; AltName: Full=Kynurenine aminotransferase IV; AltName: Full=Kynurenine--oxoglutarate transaminase 4; AltName: Full=Kynurenine--oxoglutarate transaminase IV; AltName: Full=Plasma membrane-associated fatty acid-binding protein; Short=FABPpm; AltName: Full=Transaminase A; Flags: Precursor',
                         sequence.getDescription())
        self.assertEqual("MALLHSARVL", sequence.getSequence()[:10])
        self.assertEqual(Alphabet.EXTENDED_PROTEIN_ALPHABET, sequence.getAlphabet()) # Protein

    def testImportUniprotAminoacidSequence2(self):
        """
        Import a single aminoacid sequence from UniProt
        """
        args = {'inputSequenceID': self.USERID,
                'inputSequenceName': self.NAME,
                'inputSequenceDescription': self.DESCRIPTION,
                'inputProteinSequence':
                    emprot.ProtImportSequence.IMPORT_FROM_UNIPROT,
                'uniProtSequence': self.UNIPROTID
                }
        prot20 = self.newProtocol(emprot.ProtImportSequence, **args)
        prot20.setObjLabel('20_import aminoacids,\nseq from '
                           'UniProt')
        self.launchProtocol(prot20)
        sequence = prot20.outputSequence
        self.assertEqual("UserID", sequence.getId())
        self.assertEqual('USER_SEQ', sequence.getSeqName())
        self.assertEqual('User description',
                         sequence.getDescription())
        self.assertEqual("MALLHSARVL", sequence.getSequence()[:10])
        self.assertEqual(Alphabet.EXTENDED_PROTEIN_ALPHABET, sequence.getAlphabet()) # Protein



# ***************************************************************************
# * Authors:     J.M. De la Rosa Trevin (jmdelarosa@cnb.csic.es)
# *
# *
# * This program is free software; you can redistribute it and/or modify
# * it under the terms of the GNU General Public License as published by
# * the Free Software Foundation; either version 3 of the License, or
# * (at your option) any later version.
# *
# * This program is distributed in the hope that it will be useful,
# * but WITHOUT ANY WARRANTY; without even the implied warranty of
# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# * GNU General Public License for more details.
# *
# * You should have received a copy of the GNU General Public License
# * along with this program; if not, write to the Free Software
# * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
# * 02111-1307  USA
# *
# *  All comments concerning this program package may be sent to the
# *  e-mail address 'scipion@cnb.csic.es'
# ***************************************************************************/

import os
import tempfile
try:
    from itertools import izip
except ImportError:
    izip = zip

import pyworkflow.tests as pwtests

import pwem.protocols as emprot
import pwem.objects as emobj


class TestImportBase(pwtests.BaseTest):
    @classmethod
    def setUpClass(cls):
        pwtests.setupTestProject(cls)
        cls.dsXmipp = pwtests.DataSet.getDataSet('xmipp_tutorial')
        cls.dsEmx = pwtests.DataSet.getDataSet('emx')
        cls.dsMda = pwtests.DataSet.getDataSet('mda')


class TestImportMicrographs(TestImportBase):

    def checkMicSet(self, micSet, goldFn):
        """ Compare micrographs of micSet with the 
        ones in the goldFn. Maybe except the full path.
        """
        goldSet = emobj.SetOfMicrographs(filename=goldFn)

        for mic1, mic2 in izip(goldSet, micSet):
            # Remove the absolute path in the micrographs to
            # really check that the attributes should be equal
            mic1.setFileName(os.path.basename(mic1.getFileName()))
            mic2.setFileName(os.path.basename(mic2.getFileName()))

            self.assertTrue(mic1.equalAttributes(mic2, verbose=True))

    def test_pattern(self):
        """ Import several micrographs from a given pattern.
        """
        args = {'importFrom': emprot.ProtImportMicrographs.IMPORT_FROM_FILES,
                'filesPath': self.dsXmipp.getFile('micrographs'),
                'filesPattern': '*.mrc',
                'amplitudeConstrast': 0.1,
                'sphericalAberration': 2.,
                'voltage': 100,
                'samplingRate': 2.1
                }

        def _checkOutput(prot, micsId=[], size=None):
            mics = getattr(prot, 'outputMicrographs', None)
            self.assertIsNotNone(mics)
            self.assertEqual(mics.getSize(), size)
            for i, m in enumerate(mics):
                if micsId:
                    self.assertEqual(m.getObjId(), micsId[i])
                self.assertAlmostEqual(m.getSamplingRate(), args['samplingRate'])
                a = m.getAcquisition()
                self.assertAlmostEqual(a.getVoltage(), args['voltage'])

        # Id's should be set increasing from 1 if ### is not in the 
        # pattern
        protMicImport = self.newProtocol(emprot.ProtImportMicrographs, **args)
        protMicImport.setObjLabel('from files')
        self.launchProtocol(protMicImport)
        _checkOutput(protMicImport, [1, 2, 3], size=3)

        # Id's should be taken from filename
        args['filesPattern'] = 'BPV_####.mrc'
        protMicImport = self.newProtocol(emprot.ProtImportMicrographs, **args)
        protMicImport.setObjLabel('from files (with id)')
        self.launchProtocol(protMicImport)
        _checkOutput(protMicImport, [1386, 1387, 1388], size=3)

        # Combine * and #
        args['filesPattern'] = '*_####.mrc'
        protMicImport = self.newProtocol(emprot.ProtImportMicrographs, **args)
        protMicImport.setObjLabel('from files (* with id)')
        self.launchProtocol(protMicImport)
        _checkOutput(protMicImport, [1386, 1387, 1388], size=3)

        # Id from folder
        parentFolder = tempfile.gettempdir()
        symlinkFolder = os.path.join(parentFolder, 'testId4')
        if not os.path.exists(symlinkFolder):
            os.symlink(args['filesPath'], symlinkFolder)
        args['filesPath'] = os.path.join(parentFolder, 'testId#')
        args['filesPattern'] = '*_?387.mrc'
        protMicImport = self.newProtocol(emprot.ProtImportMicrographs, **args)
        protMicImport.setObjLabel('from files (id from folder)')
        self.launchProtocol(protMicImport)
        _checkOutput(protMicImport, [4], size=1)

        # Import some micrographs from EMX        
        emxFn = self.dsEmx.getFile('coordinatesT1')
        args['importFrom'] = emprot.ProtImportMicrographs.IMPORT_FROM_EMX
        args['emxFile'] = emxFn
        protEmxImport = self.newProtocol(emprot.ProtImportMicrographs, **args)
        protEmxImport.setObjLabel('from emx (with coords)')
        self.launchProtocol(protEmxImport)
        _checkOutput(protEmxImport, [], size=1)

    def test_fromEmx(self):
        """ Import an EMX file with micrographs and defocus
        """
        emxFn = self.dsEmx.getFile('emxMicrographCtf1')
        protEmxImport = self.newProtocol(emprot.ProtImportMicrographs,
                                         importFrom=emprot.ProtImportMicrographs.IMPORT_FROM_EMX,
                                         emxFile=emxFn,
                                         magnification=10000,
                                         samplingRate=2.46
                                         )
        protEmxImport.setObjLabel('from emx ')
        self.launchProtocol(protEmxImport)

        self.checkMicSet(protEmxImport.outputMicrographs,
                         goldFn=self.dsEmx.getFile('emxMicrographCtf1Gold'))

    def test_fromXmipp(self):
        """ Import an EMX file with micrographs and defocus
        """
        micsRoot = 'xmipp_project/Micrographs/Imported/run_001/%s'
        micsMd = self.dsXmipp.getFile(micsRoot % 'micrographs.xmd')
        prot1 = self.newProtocol(emprot.ProtImportMicrographs,
                                 importFrom=emprot.ProtImportMicrographs.IMPORT_FROM_XMIPP3,
                                 mdFile=micsMd,
                                 magnification=10000,
                                 samplingRate=1.237,
                                 voltage=200,
                                 sphericalAberration=2.0
                                 )
        prot1.setObjLabel('from xmipp (no-ctf)')
        self.launchProtocol(prot1)
        self.checkMicSet(prot1.outputMicrographs,
                         goldFn=micsMd.replace('.xmd', '_gold.sqlite'))

        micsRoot = 'xmipp_project/Micrographs/Screen/run_001/%s'
        micsMd = self.dsXmipp.getFile(micsRoot % 'micrographs.xmd')
        prot2 = self.newProtocol(emprot.ProtImportMicrographs,
                                 importFrom=emprot.ProtImportMicrographs.IMPORT_FROM_XMIPP3,
                                 mdFile=micsMd,
                                 magnification=10000,
                                 samplingRate=1.237,
                                 voltage=200,
                                 sphericalAberration=2.0
                                 )
        prot2.setObjLabel('from xmipp (ctf)')
        self.launchProtocol(prot2)
        self.checkMicSet(prot2.outputMicrographs,
                         goldFn=micsMd.replace('.xmd', '_gold.sqlite'))

    def test_fromScipion(self):
        """ Import an EMX file with micrographs and defocus
        """
        micsSqlite = self.dsXmipp.getFile('micrographs/micrographs.sqlite')
        print("Importing from sqlite: ", micsSqlite)

        micSet = emobj.SetOfMicrographs(filename=micsSqlite)
        # Gold values
        # _samplingRate -> 1.237
        # _acquisition._voltage -> 300.0
        # _acquisition._magnification -> 50000.0

        for k in ['_samplingRate', '_acquisition._voltage', '_acquisition._magnification']:
            print(k, "->", micSet.getProperty(k))
        prot = self.newProtocol(emprot.ProtImportMicrographs,
                                objLabel='from scipion',
                                importFrom=emprot.ProtImportMicrographs.IMPORT_FROM_SCIPION,
                                sqliteFile=micsSqlite,
                                samplingRate=float(micSet.getProperty('_samplingRate')),
                                voltage=float(micSet.getProperty('_acquisition._voltage')),
                                magnification=int(float(micSet.getProperty('_acquisition._magnification')))
                                )

        self.launchProtocol(prot)

        micSet = getattr(prot, 'outputMicrographs', None)
        self.assertIsNotNone(micSet)

        self.assertAlmostEqual(1.237, micSet.getSamplingRate())

        acq = micSet.getAcquisition()
        self.assertAlmostEqual(300., acq.getVoltage())
        self.assertAlmostEqual(50000., acq.getMagnification())



# **************************************************************************
# *
# * Authors:     Laura del Cano (ldelcano@cnb.csic.es)
# *              Ignacio Foche (ifoche@cnb.csic.es)
# *
# * Unidad de  Bioinformatica of Centro Nacional de Biotecnologia , CSIC
# *
# * This program is free software; you can redistribute it and/or modify
# * it under the terms of the GNU General Public License as published by
# * the Free Software Foundation; either version 3 of the License, or
# * (at your option) any later version.
# *
# * This program is distributed in the hope that it will be useful,
# * but WITHOUT ANY WARRANTY; without even the implied warranty of
# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# * GNU General Public License for more details.
# *
# * You should have received a copy of the GNU General Public License
# * along with this program; if not, write to the Free Software
# * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
# * 02111-1307  USA
# *
# *  All comments concerning this program package may be sent to the
# *  e-mail address 'scipion@cnb.csic.es'
# *
# **************************************************************************
"""
In this module are protocol related to EM imports of Masks...
"""
import enum
from os.path import exists, basename

import pyworkflow.protocol.params as params
import pyworkflow.utils as pwutils

from pwem.objects import Mask, VolumeMask
from pwem.emlib.image import ImageHandler
from pwem.protocols.protocol_import.images import cleanFileName

from .base import ProtImport

class ImportMaskOutput(enum.Enum):
    outputMask = VolumeMask


class ProtImportMask(ProtImport):
    """ Class for import masks from existing files. """
    _label = 'import mask'
    _possibleOutputs = ImportMaskOutput
    
    # --------------------------- DEFINE param functions ----------------------
    def _defineParams(self, form):
        
        form.addSection(label='Import')

        form.addParam('maskPath', params.PathParam,
                      label="Mask path",
                      help="Select the file path of the mask\n")
        form.addParam('samplingRate', params.FloatParam, default=1.,
                      label=pwutils.properties.Message.LABEL_SAMP_RATE)

    def _insertAllSteps(self):
        self._insertFunctionStep('importMaskStep', self.maskPath.get(),
                                 self.samplingRate.get())

    # --------------------------- STEPS functions -----------------------------

    def importMaskStep(self, path, samplingRate):
        """ Copy mask from maskPath.
        """
        self.info("Using mask path: '%s'" % path)

        # Copy the image file into the project
        dst = self._getExtraPath(basename(path))
        dst, cleaned = cleanFileName(dst)
        pwutils.copyFile(path, dst)

        # Retrieve image dimensions
        imgh = ImageHandler()
        _, _, z, n = imgh.getDimensions(dst)

        # Create a 2D or 3D Mask, consider the case of n>1
        # as the case of some volume maps in mrc format
        if z > 1 or n > 1:
            mask = VolumeMask()
        else:
            mask = Mask()

        mask.setFileName(dst)
        mask.setSamplingRate(samplingRate)

        self._defineOutputs(**{self._possibleOutputs.outputMask.name:mask})

    # --------------------------- INFO functions ------------------------------
    def _validate(self):
        errors = []
        if not self.maskPath.hasValue():
            errors.append("Mask path cannot be empty.")
        elif not exists(self.maskPath.get()):
            errors.append("Mask not found at *%s*" % self.maskPath.get())
        if not self.samplingRate.hasValue():
            errors.append("Sampling rate cannot be empty.")
        return errors

    def _summary(self):
        summary = ['Mask file imported from *%s*' % self.maskPath.get()]
        return summary



# **************************************************************************
# *
# * Authors:     J.M. De la Rosa Trevin (jmdelarosa@cnb.csic.es)
# *
# * Unidad de  Bioinformatica of Centro Nacional de Biotecnologia , CSIC
# *
# * This program is free software; you can redistribute it and/or modify
# * it under the terms of the GNU General Public License as published by
# * the Free Software Foundation; either version 3 of the License, or
# * (at your option) any later version.
# *
# * This program is distributed in the hope that it will be useful,
# * but WITHOUT ANY WARRANTY; without even the implied warranty of
# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# * GNU General Public License for more details.
# *
# * You should have received a copy of the GNU General Public License
# * along with this program; if not, write to the Free Software
# * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
# * 02111-1307  USA
# *
# *  All comments concerning this program package may be sent to the
# *  e-mail address 'scipion@cnb.csic.es'
# *
# **************************************************************************


import pyworkflow.protocol.params as params
from pyworkflow.object import Integer
from pwem.objects import SetOfParticles

from .protocol_2d import ProtAlign2D


class ProtAlignmentAssign(ProtAlign2D):
    """ Assign the alignment calculated for a set of particles to another set.
    This protocol will take into account the differences of pixel size (A/pix)
    between the two sets and multiply by the right factor the shifts.
    The particles with the alignment can also be a subset of the other images
    """
    _label = 'assign alignment'
    OUTPUT_NAME ='outputParticles'
    IGNORE_LABEL = 'Ignore fractional shifts'
    _possibleOutputs = {OUTPUT_NAME: SetOfParticles}

    def _defineParams(self, form):
        form.addSection(label='Input')

        form.addParam('inputParticles', params.PointerParam, pointerClass=SetOfParticles,
                      label='Input particles',
                      help='Select the particles that you want to update the new alignment.')
        form.addParam('inputAlignment', params.PointerParam, pointerClass=SetOfParticles,
                      label="Input alignments",
                      help='Select the particles with alignment to be apply to the other particles.')
        form.addParam('assignRandomSubsets', params.BooleanParam, default=True,
                      expertLevel=params.LEVEL_ADVANCED,
                      label="Assign random subsets?",
                      help="If yes, the random subset information from the "
                           "assignment input will be transferred to the output "
                           "particles.")
        form.addBooleanParam('ignoreFractionalShift',
                             label=self.IGNORE_LABEL,
                             help='When extracting coordinates, the integer part of the shifts from the alignment'
                                  ' can be transferred to the 2d coordinate. The remaining decimal part is stored'
                                  ' as xFrac and yFrac in the particles. Leave this active if you want current alignment'
                                  ' shifts to be used, IGNORING the decimal fraction that might be annotated in case this'
                                  ' set comes from an "extract coordinates" and you chose to "Apply particle shifts?"',
                             default=True)

        form.addParallelSection(threads=0, mpi=0)

# --------------------------- INSERT steps functions --------------------------

    def _insertAllSteps(self):
        """for each ctf insert the steps to compare it
        """
        self._insertFunctionStep(self.createOutputStep)

    def _updateItem(self, item, row):
        """ Implement this function to do some
        update actions over each single item
        that will be stored in the output Set.
        """

        alignedParticle = self.alignmentData[item.getObjId()]
        # If alignment is found for this particle set the alignment info
        # on the output particle, if not do not write that item
        if alignedParticle is not None:
            alignment = alignedParticle.getTransform()

            # If shifts Applied before at extraction coordinate time
            if not self.ignoreFractionalShift.get() and self.hasFractions():
                coord = item.getCoordinate()

                alignment.invert()
                alignment.setShifts(-coord.xFrac.get(),
                                -coord.yFrac.get(),
                                0)
                alignment.invert()

                # remove the attribute since it has been already applied
                del coord.xFrac
                del coord.yFrac

            else:
                alignment.scaleShifts(self.scale)

            item.setTransform(alignment)

            if self.assignRandomSubsets:
                subset = \
                    alignedParticle.getAttributeValue('_rlnRandomSubset', None)
                if subset is not None:
                    item._rlnRandomSubset = Integer(subset)
        else:
            item._appendItem = False

    def createOutputStep(self):

        inputParticles = self.inputParticles.get()

        # Store data to be used in the update item
        self.alignmentData = self.inputAlignment.get()

        # Trigger the connection to avoid connecting and disconnecting later when using self.alignmentData[23]
        self.alignmentData.load()
        self.scale = self.alignmentData.getSamplingRate()/inputParticles.getSamplingRate()

        # Add alignment info from corresponding item on inputAlignment
        # Output
        outputParticles = self._createSetOfParticles()
        outputParticles.copyInfo(inputParticles)
        outputParticles.setAlignment(self.alignmentData.getAlignment())

        self.hasFractions() # Initialize here this to avoid using the set during the loop of copyItems.
        outputParticles.copyItems(inputParticles,
                                  updateItemCallback=self._updateItem)

        self._defineOutputs(**{self.OUTPUT_NAME:outputParticles})
        self._defineSourceRelation(self.inputParticles, outputParticles)
        self._defineSourceRelation(self.inputAlignment, outputParticles)

    def _summary(self):
        summary = []
        if not hasattr(self, self.OUTPUT_NAME):
            summary.append("Output particles not ready yet.")
        else:
            scale = self.inputAlignment.get().getSamplingRate()/self.inputParticles.get().getSamplingRate()
            summary.append("Assigned alignment to %s particles from a total of %s." % (
                self.outputParticles.getSize(), self.inputParticles.get().getSize()))
            if scale != 1:
                summary.append("Applied scale of %s." % scale)
        return summary

    def _methods(self):
        methods = []
        if not hasattr(self, self.OUTPUT_NAME):
            methods.append("Output particles not ready yet.")
        else:
            scale = self.inputAlignment.get().getSamplingRate()/self.inputParticles.get().getSamplingRate()
            methods.append("We assigned alignment to %s particles from %s and produced %s."
                           % (self.outputParticles.getSize(), self.getObjectTag('inputParticles'),
                              self.getObjectTag('outputParticles')))
            if scale != 1:
                methods.append("Applied scale factor of %s." % scale)
        return methods

    def _validate(self):
        """ The function of this hook is to add some validation before the protocol
        is launched to be executed. It should return a list of errors. If the list is
        empty the protocol can be executed.
        """
        # check that input set of aligned particles do have 2D alignment
        errors = []
        inputAlignmentSet = self.inputAlignment.get()
        if not inputAlignmentSet.hasAlignment():
            errors.append("Input alignment set should contains some kind of alignment (2D, 3D or Projection).")
        else:
            # Just for consistency, check that the particles really contains Transform object
            first = inputAlignmentSet.getFirstItem()
            alignment = first.getTransform()
            if alignment is None:
                errors.append('Inconsistency detected in *Input alignment* !!!')
                errors.append('It has alignment: _%s_, but the alignment is missing!!!' %
                              inputAlignmentSet.getAlignment())
            
        # Add some errors if input is not valid
        return errors
    def hasFractions(self):

        if not hasattr(self, '_hasFractions'):
            item = self.inputParticles.get().getFirstItem()
            self._hasFractions = item.hasCoordinate() and hasattr(item.getCoordinate(), "xFrac")

        return self._hasFractions

    def _warnings(self):
        warnings = []
        if self.hasFractions() and self.ignoreFractionalShift.get():
            warnings.append("Your set has residual shifts from a previous 'extract coordinates' protocol."
                            " And will be ignored. If you want to use them, cancel this and deactivate '%s'."
                            % self.IGNORE_LABEL)

        return warnings



# **************************************************************************
# *
# * Authors:     roberto Marabini (roberto@cnb.csic.es)
# *
# * Unidad de  Bioinformatica of Centro Nacional de Biotecnologia , CSIC
# *
# * This program is free software; you can redistribute it and/or modify
# * it under the terms of the GNU General Public License as published by
# * the Free Software Foundation; either version 3 of the License, or
# * (at your option) any later version.
# *
# * This program is distributed in the hope that it will be useful,
# * but WITHOUT ANY WARRANTY; without even the implied warranty of
# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# * GNU General Public License for more details.
# *
# * You should have received a copy of the GNU General Public License
# * along with this program; if not, write to the Free Software
# * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
# * 02111-1307  USA
# *
# *  All comments concerning this program package may be sent to the
# *  e-mail address 'scipion@cnb.csic.es'
# *
# **************************************************************************
#
from copy import deepcopy
from tempfile import NamedTemporaryFile
from collections import Counter
from urllib.request import urlretrieve

import numpy as np
import os

from pyworkflow.tests import *

import pwem.convert as emconv
from pwem.emlib.image import ImageHandler
from pwem.protocols import ProtImportPdb


class TestAtomicStructHandler(unittest.TestCase):
    """test atomicstructure handler"""
    def testIntToChain(self):
        aSH = emconv.AtomicStructHandler(self.PDBFileName)
        solString = ['A',  'B',  'C',  'D',  'E',  'F',  'G',
                     'H',  'I',  'J',  'K',  'L',  'M',  'N',
                     'O',  'P',  'Q',  'R',  'S',  'T',  'U',
                     'V',  'W',  'X',  'Y',  'Z',  '0',  '1',
                     '2',  '3',  '4',  '5',  '6',  '7',  '8',
                     '9',  'a',  'b',  'c',  'd',  'e',  'f',
                     'g',  'h',  'i',  'j',  'k',  'l',  'm',
                     'n',  'o',  'p',  'q',  'r',  's',  't',
                     'u',  'v',  'w',  'x',  'y',  'z', "AA"]
        for i in range(63):
            self.assertEqual(solString[i], aSH._intToChain(i))

    def testReadPDB(self):
        aSH = emconv.AtomicStructHandler(self.PDBFileName)
        structure = aSH.getStructure()

        solDict = {}
        solDict['structure_method'] = 'x-ray diffraction'
        solDict['head'] = 'extracellular matrix'
        solDict['name'] = 'x-ray crystallographic determination ' \
                          'of a collagen-like peptide with the ' \
                          'repeating sequence (pro-pro-gly)'
        solDict['author'] = 'R.Z.Kramer,L.Vitagliano,J.Bella,R.Berisio,' \
                            'L.Mazzarella,B.Brodsky,A.Zagari,H.M.Berman'
        solDict['deposition_date'] = '1998-01-22'
        for k, v in solDict.items():
            self.assertEqual(structure.header[k].strip(), v)

        solList = ['N', 'CA', 'C', 'O', 'CB']
        counter = 0
        for atom in structure.get_atoms():
            self.assertEqual(solList[counter], atom.get_name())
            counter += 1

    def testReadCIF(self):
        aSH = emconv.AtomicStructHandler(self.CIFFileName)
        structure = aSH.getStructure()

        solList = ['N', 'CA', 'C', 'O', 'CB']
        counter = 0
        for atom in structure.get_atoms():
            self.assertEqual(solList[counter], atom.get_name())
            counter += 1

        solDict = {}
        solDict['_exptl.method'] = 'x-ray diffraction'
        solDict['_struct_keywords.pdbx_keywords'] = 'extracellular matrix'
        solDict['_struct.title'] = 'x-ray crystallographic determination ' \
                                   'of a collagen-like peptide with the ' \
                                   'repeating sequence (pro-pro-gly)'
        _dict = aSH.readLowLevel(self.CIFFileName)

        for k, v in solDict.items():
            self.assertEqual(_dict[k][0].strip().lower(), v.lower())

    def testRenameToChains(self):
        aSH = emconv.AtomicStructHandler(self.PDBFileName)
        structure = aSH.getStructure()

        model = structure[0]
        chain = model['B']
        chain.id = 'CC'
        aSH.renameChains(structure)
        for chain in structure.get_chains():
            self.assertEqual(chain.id, 'C')

    def testWritePDB(self):
        aSH = emconv.AtomicStructHandler(self.PDBFileName)
        PDBFileName2 = self.PDBFileName.replace(".pdb", "_2.pdb")
        aSH._write(PDBFileName2)
        aSH2 = emconv.AtomicStructHandler(PDBFileName2)
        structure1 = aSH.getStructure()
        structure2 = aSH2.getStructure()

        for atom1, atom2 in zip(structure1.get_atoms(),
                                structure2.get_atoms()):
            self.assertEqual(atom1.get_name(), atom2.get_name())
        os.unlink(PDBFileName2)

    def testWriteCIF(self):
        aSH = emconv.AtomicStructHandler(self.PDBFileName)
        CIFFileName2 = self.CIFFileName.replace(".cif", "_2.cif")
        aSH._write(CIFFileName2)
        aSH2 = emconv.AtomicStructHandler(CIFFileName2)
        structure1 = aSH.getStructure()
        structure2 = aSH2.getStructure()

        for atom1, atom2 in zip(structure1.get_atoms(),
                                structure2.get_atoms()):
            self.assertEqual(atom1.get_name(), atom2.get_name())
        os.unlink(CIFFileName2)

    def testCIFToPDB(self):
        aSH = emconv.AtomicStructHandler(self.CIFFileName)
        structure1 = aSH.getStructure()
        PDBFileName2 = self.PDBFileName.replace(".pdb", "_2.pdb")
        aSH.write(PDBFileName2)
        aSH.read(PDBFileName2)
        structure2 = aSH.getStructure()
        for atom1, atom2 in zip(structure1.get_atoms(),
                                structure2.get_atoms()):
            self.assertEqual(atom1.get_name(), atom2.get_name())
        os.unlink(PDBFileName2)

    def testPDBToCIF(self):
        aSH1 = emconv.AtomicStructHandler(self.PDBFileName)
        CIFFileName2 = self.CIFFileName.replace(".cif", "_2.cif")
        aSH1.write(CIFFileName2)
        aSH2 = emconv.AtomicStructHandler(CIFFileName2)

        structure1 = aSH1.getStructure()
        structure2 = aSH2.getStructure()
        for atom1, atom2 in zip(structure1.get_atoms(),
                                structure2.get_atoms()):
            self.assertEqual(atom1.get_name(), atom2.get_name())
        os.unlink(CIFFileName2)

    def testCenterOfMass(self):
        aSH = emconv.AtomicStructHandler()
        structure = aSH.read(self.CIFFileName)
        x, y, z = aSH.centerOfMass(structure)
        self.assertAlmostEqual(x,  8.1593891597, 2)
        self.assertAlmostEqual(y, 21.1833304818, 2)
        self.assertAlmostEqual(z, 20.0177924407, 2)

    def testTransformTranslation(self):
        aSHSource = emconv.AtomicStructHandler(self.PDBFileName)
        structure = aSHSource.getStructure()
        structure_copy = deepcopy(aSHSource.getStructure())
        shift = [100., 50., 25.]
        #        rotation_matrix = euler_matrix(deg2rad(45.), 0., 0., 'szyz')
        rotation_matrix = emconv.euler_matrix(0., 0., 0., 'szyz')
        translation = emconv.translation_matrix(shift)
        M = emconv.concatenate_matrices(rotation_matrix, translation)
        aSHSource.transform(M)
        for atom1, atom2 in zip(structure.get_atoms(),
                                structure_copy.get_atoms()):
            coord1 = atom1.get_coord()
            coord2 = [sum(x) for x in zip(atom2.get_coord(), shift)]
            for i in range(3):
                self.assertAlmostEqual(coord1[i], coord2[i], 2)

    def testTransformRotation(self):
        aSHSource = emconv.AtomicStructHandler(self.PDBFileName)
        structure = aSHSource.getStructure()
        structure_copy = deepcopy(structure)
        rot = np.deg2rad(10)
        theta = np.deg2rad(20.)
        psi = np.deg2rad(30.)
        rotation_matrix = emconv.euler_matrix(rot, theta, psi, 'szyz')
        translation = emconv.translation_matrix([0., 0., 0.])
        M = emconv.concatenate_matrices(rotation_matrix, translation)
        aSHSource.transform(M)
        m = M[:3, :3]
        for atom1, atom2 in zip(structure.get_atoms(),
                                structure_copy.get_atoms()):
            coord1 = atom1.get_coord()
            coord2 = m.dot(atom2.get_coord())
            for i in range(3):
                self.assertAlmostEqual(coord1[i], coord2[i], 2)

    def testTransformRotationAndTranslation(self):
        aSHSource = emconv.AtomicStructHandler(self.PDBFileName)
        structure = aSHSource.getStructure()
        structure_copy = deepcopy(structure)
        rot = np.deg2rad(10)
        theta = np.deg2rad(20.)
        psi = np.deg2rad(30.)
        rotation_matrix = emconv.euler_matrix(rot, theta, psi, 'szyz')
        shift = [100., 50., 25.]
        translation = emconv.translation_matrix(shift)
        M = emconv.concatenate_matrices(rotation_matrix, translation)
        aSHSource.transform(M)
        m = M[:3, :3]
        for atom1, atom2 in zip(structure.get_atoms(),
                                structure_copy.get_atoms()):
            coord1 = atom1.get_coord()
            coord2 = atom2.get_coord()
            coord2 = [sum(x) for x in zip(coord2, shift)]
            coord2 = m.dot(coord2)
            for i in range(3):
                self.assertAlmostEqual(coord1[i], coord2[i], 2)

    def testTransformTranslationCoherence(self):
        """
        Question: If I transform the PDB and the 3D map with matrix T
        do they move in the same direction?
        I do not know how to make an automatic test to check this
        The following code perform all operations but check that
        the PDB and 3D map match. This should be check using
        your eye.

        change False to True (in the following if) to
        perform all or some of the checks
        """

        # retrieve "Structure of the human TRPC3
        # both 3Dmap and PDB

        doTest = False

        if not doTest:

            print("This test is to be tested manually since it opens chimera afterwards")
            print("For testing this, edit this file and set doTest = True")
            return

        PDBID = '6CUD'
        EMDBID = '7620'

        doAll = False

        if False or doAll:  # set to False if you aready have the 3dmap file
            url = 'ftp://ftp.ebi.ac.uk/pub/databases/emdb/structures/EMD-%s/map/emd_%s.map.gz' % \
                  (EMDBID, EMDBID)
            urlretrieve(url, 'emd_%s.map.gz' % EMDBID)
            os.system("gunzip emd_%s.map.gz" % EMDBID)  # file is gzipped
        if False or doAll:  # set to False if you aready have the PDB file
            aSH = emconv.AtomicStructHandler()
            pdbFileName = aSH.readFromPDBDatabase(PDBID, type='pdb',
                                                  dir=os.getcwd())
        else:
            pdbFileName = 'pdb%s.ent' % PDBID.lower()

        # get 3D map sampling
        from pwem.convert.headers import Ccp4Header
        header = Ccp4Header("emd_%s.map" % EMDBID, readHeader=True)
        sampling, y, z = header.getSampling()

        def __runXmippProgram(program, args):
            """ Internal function to launch a Xmipp program. """
            from pwem import Domain
            xmipp3 = Domain.importFromPlugin('xmipp3', doRaise=True)
            xmipp3.runXmippProgram(program, args)

        def __getXmippEulerAngles(matrix):
            """ Internal fuction to convert scipion to xmipp angles"""
            from pwem import Domain
            geometryFromMatrix = Domain.importFromPlugin('xmipp3.convert',
                                                         'geometryFromMatrix',
                                                         doRaise=True)

            return geometryFromMatrix(matrix, False)

        def __applyTransform(suffix, pdbFileName, shift, angles, sampling):
            """ auxiliary function, transform PDB and 3dmap files"""
            # create a Scipion transformation matrix
            from numpy import deg2rad
            rotation_matrix = emconv.euler_matrix(deg2rad(angles[0]),
                                                  deg2rad(angles[1]),
                                                  deg2rad(angles[2]), 'szyz')
            translation = emconv.translation_matrix(shift)
            M = emconv.concatenate_matrices(rotation_matrix, translation)

            # apply it to the pdb file
            # if rotation move to center
            aSH = emconv.AtomicStructHandler(pdbFileName)
            if angles[0] != 0. or angles[1] != 0. or angles[2] != 0.:
                ih = ImageHandler()
                x, y, z, n = ih.getDimensions("emd_%s.map" % EMDBID)
                x /= 2.
                y /= 2.
                z /= 2.
                localShift = [-x, -y, -z]
                rotation_matrix = emconv.euler_matrix(0., 0., 0., 'szyz')
                translation = emconv.translation_matrix(localShift)
                localM = emconv.concatenate_matrices(rotation_matrix, translation)
                aSH.transform(localM, sampling=sampling)

            aSH.transform(M, sampling=sampling)

            if angles[0] != 0. or angles[1] != 0. or angles[2] != 0.:
                localShift = [x, y, z]
                rotation_matrix = emconv.euler_matrix(0., 0., 0., 'szyz')
                translation = emconv.translation_matrix(localShift)
                localM = emconv.concatenate_matrices(rotation_matrix, translation)
                aSH.transform(localM, sampling=sampling)

            aSH.write("%s_%s_transformed.ent" % (suffix, PDBID.lower()))

            # get equivalent xmipp transformation
            shift, angles = __getXmippEulerAngles(M)
            # shift 3map and set sampling
            __runXmippProgram("xmipp_transform_geometry",
                              '-i emd_%s.map '
                              '-o %s_emd_%s_transform.map '
                              '--interp linear '
                              '--shift %f %f %f '
                              '--rotate_volume euler %f %f %f ' % (
                                  EMDBID,
                                  suffix,
                                  EMDBID,
                                  shift[0], shift[1], shift[2],
                                  angles[0], angles[1], angles[2]
                              )
                              )
            header = Ccp4Header("%s_emd_%s_transform.map" % (suffix, EMDBID),
                                readHeader=True)
            header.setSampling(sampling)
            # put the sampling back, xmipp_transform_geometry erased it
            header.writeHeader()

            # view the results with chimera
            from pwem.viewers import Chimera
            args = "%s %s %s %s" % (
                   pdbFileName,
                   "emd_%s.map" % EMDBID,
                   "%s_%s_transformed.ent" % (suffix, PDBID.lower()),
                   "%s_emd_%s_transform.map" % (suffix, EMDBID)
            )
            Chimera.runProgram(args)

        # shift atomic structure
        doAll = True
        if False or doAll:
            shift = [20., 0., 0.]
            angles = [0., 0., 0.]
            __applyTransform("Xshift", pdbFileName, shift, angles, sampling)

        # repeat test this time  rotation one angle
        # problem, xmipp rotates with respect the volume center
        # pdb with respect the origin of coordinates (much better convention)
        # in order to compare both I need to
        # move pdb to origin, rotate it, put it back in the position
        if False or doAll:
            shift = [0., 0., 0.]
            angles = [30., 0., 0.]
            __applyTransform("Rot2D", pdbFileName, shift, angles, sampling)

        # repeat test this time  rotation in 3 angles
        # problem, xmipp rotates with respect the volume center
        # pdb with respect the origin of coordinates (much better convention)
        if False or doAll:
            shift = [0., 0., 0.]
            angles = [10., 20., 30.]
            __applyTransform("Rot3D", pdbFileName, shift, angles, sampling)

        # repeat test this time  rotation in 3 angles and shift
        # problem, xmipp rotates with respect the volume center
        # pdb with respect the origin of coordinates (much better convention)
        if False or doAll:
            shift = [5., 10., 15.]
            angles = [10., 20., 30.]
            __applyTransform("Rot3DShift", pdbFileName,
                             shift, angles, sampling)

    def testReadFromPDBDatabase(self):
        PDBID = '6CUD'
        aSH = emconv.AtomicStructHandler()
        # EMD-7620
        fileName = aSH.readFromPDBDatabase(PDBID, type='pdb', dir='/tmp')
        self.assertTrue(os.path.exists(fileName))

        os.unlink(fileName)

    def testFunctionAddStructNoNewModel(self):
        """ add two atomic structures with overlapping chain ids"""
        pdbID1 = '1P30'  # A
        pdbID2 = '1CJD'  # A, B, C
        outFile = "/tmp/nomodel.cif"  # A, A002, B, C
        aSH1 = emconv.AtomicStructHandler()
        aSH2 = emconv.AtomicStructHandler()
        #
        fileName1 = aSH1.readFromPDBDatabase(pdbID1, type='mmCif', dir='/tmp')
        fileName2 = aSH2.readFromPDBDatabase(pdbID2, type='mmCif', dir='/tmp')
        aSH1.read(fileName1)
        aSH2.read(fileName2)
        atomsNum1 = len([atom.id for atom in aSH1.getStructure().get_atoms()])
        atomsNum2 = len([atom.id for atom in aSH2.getStructure().get_atoms()])
        #
        aSH1.addStruct(fileName2, outPDBfileName=outFile, useModel=False)
        chains = [chain.id for chain in aSH1.getStructure().get_chains()]
        # compare unordered lists of chains
        goal = ['A', 'A002', 'B', 'C']
        self.assertTrue(Counter(chains) == Counter(goal),
                        "{} != {}".format(chains, goal))

        atomsNumT = len([atom.id for atom in aSH1.getStructure().get_atoms()])
        self.assertEqual(atomsNum1 + atomsNum2, atomsNumT)
        os.unlink(fileName1)
        os.unlink(fileName2)
        os.unlink(outFile)

    def testFunctionAddStructNoNewModelAddTwice(self):
        """ add two atomic structures with overlapping chain ids, last atomic
        structure is added two times"""
        pdbID1 = '1P30'  # A,
        pdbID2 = '1CJD'  # A, B, C
        outFile = "/tmp/nomodel.cif"  # A, A002, B, C, A003, B002, C002
        aSH1 = emconv.AtomicStructHandler()
        aSH2 = emconv.AtomicStructHandler()
        #
        fileName1 = aSH1.readFromPDBDatabase(pdbID1, type='mmCif', dir='/tmp')
        fileName2 = aSH2.readFromPDBDatabase(pdbID2, type='mmCif', dir='/tmp')
        aSH1.read(fileName1)
        aSH2.read(fileName2)
        atomsNum1 = len([atom.id for atom in aSH1.getStructure().get_atoms()])
        atomsNum2 = len([atom.id for atom in aSH2.getStructure().get_atoms()])
        #
        aSH1.addStruct(fileName2, outPDBfileName=outFile, useModel=False)
        aSH1.addStruct(fileName2, outPDBfileName=outFile, useModel=False)
        chains = [chain.id for chain in aSH1.getStructure().get_chains()]
        # compare unordered lists of chains
        goal = ['A', 'A002', 'B', 'C', 'A003', 'B002', 'C002']
        self.assertTrue(Counter(chains) == Counter(goal),
                        "{} != {}".format(chains, goal))

        atomsNumT = len([atom.id for atom in aSH1.getStructure().get_atoms()])
        self.assertEqual(atomsNum1 + atomsNum2 + atomsNum2, atomsNumT)
        os.unlink(fileName1)
        os.unlink(fileName2)
        os.unlink(outFile)

    def testFunctionAddStructNewModel(self):
        pdbID1 = '1P30'  # A
        pdbID2 = '1CJD'  # A, B,C
        outFile = "/tmp/model.cif"
        aSH1 = emconv.AtomicStructHandler()
        aSH2 = emconv.AtomicStructHandler()
        #
        fileName1 = aSH1.readFromPDBDatabase(pdbID1, type='mmCif', dir='/tmp')
        fileName2 = aSH2.readFromPDBDatabase(pdbID2, type='mmCif', dir='/tmp')
        aSH1.read(fileName1)
        aSH2.read(fileName2)
        atomsNum1 = len([atom.id for atom in aSH1.getStructure().get_atoms()])
        atomsNum2 = len([atom.id for atom in aSH2.getStructure().get_atoms()])
        #
        aSH1.addStruct(fileName2, outPDBfileName=outFile, useModel=True)
        #
        # aSH1.addStruct(fileName2, outPDBfileName=outFile, useModel=False)
        chains = [chain.id for chain in aSH1.getStructure().get_chains()]
        # compare unordered lists of chains
        goal = ['A', 'A', 'B', 'C']
        self.assertTrue(Counter(chains) == Counter(goal),
                        "{} != {}".format(chains, goal))
        atomsNumT = len([atom.id for atom in aSH1.getStructure().get_atoms()])
        self.assertEqual(atomsNum1 + atomsNum2, atomsNumT)

        os.unlink(fileName1)
        os.unlink(fileName2)
        os.unlink(outFile)

    def testFunctionSelectChain(self):
        pdbID1 = '1P30'  # A,B,C
        outFile = "/tmp/model.cif"
        aSH1 = emconv.AtomicStructHandler()
        aSH2 = emconv.AtomicStructHandler()
        #
        fileName1 = aSH1.readFromPDBDatabase(pdbID1, type='mmCif', dir='/tmp')
        aSH1.read(fileName1)
        atomsNum1 = len([atom.id for atom in aSH1.getStructure().get_atoms()])
        #
        chainID = 'A'
        outFileName = "/tmp/output.mmcif"
        aSH1.extractChain(chainID=chainID, modelID='0', end=20,
                          filename=outFileName)
        chains = [chain.id for chain in aSH1.getStructure().get_chains()]
        # compare unordered lists of chains
        goal = chainID
        self.assertTrue(Counter(chains) == Counter(goal),
                        "{} != {}".format(chains, goal))
        aSH2.read(outFileName)
        atomsNumT = len([atom.id for atom in aSH2.getStructure().get_atoms()])
        self.assertEqual(122, atomsNumT)

        # os.unlink(fileName1)
        # os.unlink(outFile)

    @classmethod
    def tearDownClass(cls):
        if os.path.exists(cls.PDBFileName):
            os.unlink(cls.PDBFileName)
        if os.path.exists(cls.CIFFileName):
            os.unlink(cls.CIFFileName)

    @classmethod
    def setUpClass(cls):
        PDBString = """HEADER    EXTRACELLULAR MATRIX                    22-JAN-98   1A3I
TITLE     X-RAY CRYSTALLOGRAPHIC DETERMINATION OF A COLLAGEN-LIKE
TITLE    2 PEPTIDE WITH THE REPEATING SEQUENCE (PRO-PRO-GLY)
EXPDTA    X-RAY DIFFRACTION
AUTHOR    R.Z.KRAMER,L.VITAGLIANO,J.BELLA,R.BERISIO,L.MAZZARELLA,
AUTHOR   2 B.BRODSKY,A.ZAGARI,H.M.BERMAN
REMARK 350 BIOMOLECULE: 1
REMARK 350 APPLY THE FOLLOWING TO CHAINS: B
REMARK 350   BIOMT1   1  1.000000  0.000000  0.000000        0.00000
REMARK 350   BIOMT2   1  0.000000  1.000000  0.000000        0.00000
ATOM      1  N   PRO B   1       8.316  21.206  21.530  1.00 17.44           N
ATOM      2  CA  PRO B   1       7.608  20.729  20.336  1.00 17.44           C
ATOM      3  C   PRO B   1       8.487  20.707  19.092  1.00 17.44           C
ATOM      4  O   PRO B   1       9.466  21.457  19.005  1.00 17.44           O
ATOM      5  CB  PRO B   1       6.460  21.723  20.211  1.00 22.26           C
END"""
        f = NamedTemporaryFile(delete=False, suffix=".pdb")
        f.write(PDBString.encode('utf8'))
        f.close()
        cls.PDBFileName = f.name
        CIFString = """data_1A3I
#
loop_
_audit_author.name
_audit_author.pdbx_ordinal
'Kramer, R.Z.'   1
'Vitagliano, L.' 2
'Bella, J.'      3
'Berisio, R.'    4
'Mazzarella, L.' 5
'Brodsky, B.'    6
'Zagari, A.'     7
'Berman, H.M.'   8
#
_database.entry_id        1A3I
_database.pdbx_code_NDB   1A3I
_database.pdbx_code_PDB   1A3I
_database.code_CSD        ?
#
loop_
_database_2.database_id
_database_2.database_code
PDB  1A3I
NDB  1A3I
RCSB 1A3I
#
_struct_keywords.entry_id        1A3I
_struct_keywords.pdbx_keywords   'EXTRACELLULAR MATRIX'
_struct_keywords.text            ?
#data_1A3I
#
loop_
_audit_author.name
_audit_author.pdbx_ordinal
'Kramer, R.Z.'   1
'Vitagliano, L.' 2
'Bella, J.'      3
'Berisio, R.'    4
'Mazzarella, L.' 5
'Brodsky, B.'    6
'Zagari, A.'     7
'Berman, H.M.'   8
#
_database.entry_id        1A3I
_database.pdbx_code_NDB   1A3I
_database.pdbx_code_PDB   1A3I
_database.code_CSD        ?
#
loop_
_database_2.database_id
_database_2.database_code
PDB  1A3I
NDB  1A3I
RCSB 1A3I
#
_struct_keywords.entry_id        1A3I
_struct_keywords.pdbx_keywords   'EXTRACELLULAR MATRIX'
_struct_keywords.text            ?
#
_exptl.entry_id          1A3I
_exptl.method            'X-RAY DIFFRACTION'
_exptl.crystals_number   ?
#
_struct.entry_id                  1A3I
_struct.title                     'X-RAY CRYSTALLOGRAPHIC DETERMINATION OF A COLLAGEN-LIKE PEPTIDE WITH THE REPEATING SEQUENCE (PRO-PRO-GLY)'
_struct.pdbx_descriptor           'EXTRACELLULAR MATRIX'
_struct.pdbx_model_details        ?
_struct.pdbx_details              ?
_struct.pdbx_CASP_flag            ?
_struct.pdbx_model_type_details   ?
#
_struct_biol_gen.biol_id                        1
_struct_biol_gen.asym_id                        A
_struct_biol_gen.pdbx_new_asym_id               A
_struct_biol_gen.pdbx_new_pdb_asym_id           B
_struct_biol_gen.symmetry                       1_555
_struct_biol_gen.pdbx_before_begin_residue_no   1
_struct_biol_gen.pdbx_before_end_residue_no     1
_struct_biol_gen.pdbx_after_begin_residue_no    1
_struct_biol_gen.pdbx_after_end_residue_no      1
_struct_biol_gen.details                        ?
_struct_biol_gen.pdbx_full_symmetry_operation   x,y,z
_struct_biol_gen.pdbx_PDB_order                 1
#
_struct_biol_view.biol_id            ?
_struct_biol_view.id                 ?
_struct_biol_view.rot_matrix[1][1]   ?
_struct_biol_view.rot_matrix[1][2]   ?
_struct_biol_view.rot_matrix[1][3]   ?
_struct_biol_view.rot_matrix[2][1]   ?
_struct_biol_view.rot_matrix[2][2]   ?
_struct_biol_view.rot_matrix[2][3]   ?
_struct_biol_view.rot_matrix[3][1]   ?
_struct_biol_view.rot_matrix[3][2]   ?
_struct_biol_view.rot_matrix[3][3]   ?
_struct_biol_view.pdbx_vector[1]     ?
_struct_biol_view.pdbx_vector[2]     ?
_struct_biol_view.pdbx_vector[3]     ?
#
_struct_asym.id                                              A
_struct_asym.pdbx_PDB_id                                     B
_struct_asym.pdbx_alt_id                                     B
_struct_asym.pdbx_blank_PDB_chainid_flag                     N
_struct_asym.pdbx_type                                       HETAIN
_struct_asym.pdbx_order                                      1
_struct_asym.pdbx_modified                                   N
_struct_asym.pdbx_fraction_per_asym_unit                     ?
_struct_asym.entity_id                                       1
_struct_asym.pdbx_missing_num_begin_of_chain_not_in_seqres   ?
_struct_asym.pdbx_missing_num_end_of_chain_not_in_seqres     ?
_struct_asym.pdbx_missing_num_begin_of_chain_in_seqres       ?
_struct_asym.details                                         ?
#
_pdbx_inhibitor_info.id                  1
_pdbx_inhibitor_info.name                PROLINE
_pdbx_inhibitor_info.num_per_asym_unit   1
#
_pdbx_nonpoly_scheme.asym_id         A
_pdbx_nonpoly_scheme.entity_id       1
_pdbx_nonpoly_scheme.mon_id          PRO
_pdbx_nonpoly_scheme.ndb_seq_num     1
_pdbx_nonpoly_scheme.pdb_seq_num     1
_pdbx_nonpoly_scheme.auth_seq_num    1
_pdbx_nonpoly_scheme.pdb_mon_id      PRO
_pdbx_nonpoly_scheme.auth_mon_id     PRO
_pdbx_nonpoly_scheme.pdb_strand_id   B
_pdbx_nonpoly_scheme.pdb_ins_code    .
#
_entity.id                         1
_entity.type                       non-polymer
_entity.src_method                 syn
_entity.pdbx_description           PROLINE
_entity.formula_weight             115.130
_entity.pdbx_number_of_molecules   1
_entity.details                    ?
#
_entity_keywords.entity_id               1
_entity_keywords.text                    ?
_entity_keywords.pdbx_mutation           ?
_entity_keywords.pdbx_fragment           ?
_entity_keywords.pdbx_antibody_isotype   ?
_entity_keywords.pdbx_ec                 ?
#
_entity_name_com.entity_id   1
_entity_name_com.name        ?
#
_entity_name_sys.entity_id   1
_entity_name_sys.name        ?
#
_chem_comp.id               PRO
_chem_comp.type             'L-peptide linking'
_chem_comp.mon_nstd_flag    y
_chem_comp.name             PROLINE
_chem_comp.pdbx_synonyms    ?
_chem_comp.formula          'C5 H9 N O2'
_chem_comp.formula_weight   115.130
#
_pdbx_database_status.status_code                        ?
_pdbx_database_status.entry_id                           1A3I
_pdbx_database_status.ndb_tid                            ?
_pdbx_database_status.status_coordinates_in_NDB          ?
_pdbx_database_status.recvd_deposit_form                 ?
_pdbx_database_status.date_deposition_form               ?
_pdbx_database_status.recvd_coordinates                  ?
_pdbx_database_status.date_coordinates                   ?
_pdbx_database_status.recvd_struct_fact                  ?
_pdbx_database_status.date_struct_fact                   ?
_pdbx_database_status.recvd_internal_approval            ?
_pdbx_database_status.recvd_nmr_constraints              ?
_pdbx_database_status.date_nmr_constraints               ?
_pdbx_database_status.recvd_manuscript                   ?
_pdbx_database_status.date_manuscript                    ?
_pdbx_database_status.name_depositor                     ?
_pdbx_database_status.rcsb_annotator                     ?
_pdbx_database_status.recvd_author_approval              ?
_pdbx_database_status.date_author_approval               ?
_pdbx_database_status.recvd_initial_deposition_date      ?
_pdbx_database_status.date_submitted                     ?
_pdbx_database_status.author_approval_type               ?
_pdbx_database_status.author_release_status_code         ?
_pdbx_database_status.date_revised                       ?
_pdbx_database_status.revision_id                        ?
_pdbx_database_status.replaced_entry_id                  ?
_pdbx_database_status.revision_description               ?
_pdbx_database_status.date_of_NDB_release                ?
_pdbx_database_status.date_released_to_PDB               ?
_pdbx_database_status.date_of_PDB_release                ?
_pdbx_database_status.date_hold_coordinates              ?
_pdbx_database_status.date_hold_struct_fact              ?
_pdbx_database_status.hold_for_publication               ?
_pdbx_database_status.date_hold_nmr_constraints          ?
_pdbx_database_status.dep_release_code_coordinates       ?
_pdbx_database_status.dep_release_code_struct_fact       ?
_pdbx_database_status.dep_release_code_nmr_constraints   ?
_pdbx_database_status.dep_release_code_sequence          ?
_pdbx_database_status.pdb_date_of_author_approval        ?
_pdbx_database_status.deposit_site                       ?
_pdbx_database_status.process_site                       ?
_pdbx_database_status.skip_PDB_REMARK                    ?
#
_pdbx_database_proc.entry_id           1A3I
_pdbx_database_proc.cycle_id           ?
_pdbx_database_proc.date_begin_cycle   ?
_pdbx_database_proc.date_end_cycle     ?
_pdbx_database_proc.details            ?
#
_database_PDB_remark.id     350
_database_PDB_remark.text
;BIOMOLECULE: 1
APPLY THE FOLLOWING TO CHAINS: B
  BIOMT1   1  1.000000  0.000000  0.000000        0.00000
  BIOMT2   1  0.000000  1.000000  0.000000        0.00000
;
#
_pdbx_coord.entry_id                1A3I
_pdbx_coord.chain_atoms_Y_P         ?
_pdbx_coord.hydrogen_atoms_Y_N      ?
_pdbx_coord.solvent_atoms_Y_N       ?
_pdbx_coord.structure_factors_Y_N   ?
#
_exptl_crystal.id                    1
_exptl_crystal.density_meas          ?
_exptl_crystal.density_Matthews      ?
_exptl_crystal.density_percent_sol   ?
_exptl_crystal.description           ?
#
_symmetry.entry_id                         1A3I
_symmetry.space_group_name_H-M             'P 1'
_symmetry.pdbx_full_space_group_name_H-M   ?
_symmetry.cell_setting                     ?
_symmetry.Int_Tables_number                ?
#
_cell.entry_id           1A3I
_cell.length_a           1.000
_cell.length_b           1.000
_cell.length_c           1.000
_cell.angle_alpha        90.00
_cell.angle_beta         90.00
_cell.angle_gamma        90.00
_cell.Z_PDB              1
_cell.pdbx_unique_axis   ?
#
_database_PDB_matrix.entry_id          1A3I
_database_PDB_matrix.origx[1][1]       1.000000
_database_PDB_matrix.origx[1][2]       0.000000
_database_PDB_matrix.origx[1][3]       0.000000
_database_PDB_matrix.origx[2][1]       0.000000
_database_PDB_matrix.origx[2][2]       1.000000
_database_PDB_matrix.origx[2][3]       0.000000
_database_PDB_matrix.origx[3][1]       0.000000
_database_PDB_matrix.origx[3][2]       0.000000
_database_PDB_matrix.origx[3][3]       1.000000
_database_PDB_matrix.origx_vector[1]   0.00000
_database_PDB_matrix.origx_vector[2]   0.00000
_database_PDB_matrix.origx_vector[3]   0.00000
#
_atom_sites.entry_id                    1A3I
_atom_sites.cartn_transform_axes        ?
_atom_sites.fract_transf_matrix[1][1]   1.000000
_atom_sites.fract_transf_matrix[1][2]   0.000000
_atom_sites.fract_transf_matrix[1][3]   0.000000
_atom_sites.fract_transf_matrix[2][1]   0.000000
_atom_sites.fract_transf_matrix[2][2]   1.000000
_atom_sites.fract_transf_matrix[2][3]   0.000000
_atom_sites.fract_transf_matrix[3][1]   0.000000
_atom_sites.fract_transf_matrix[3][2]   0.000000
_atom_sites.fract_transf_matrix[3][3]   1.000000
_atom_sites.fract_transf_vector[1]      0.00000
_atom_sites.fract_transf_vector[2]      0.00000
_atom_sites.fract_transf_vector[3]      0.00000
#
_struct_ncs_oper.id             ?
_struct_ncs_oper.code           ?
_struct_ncs_oper.details        ?
_struct_ncs_oper.matrix[1][1]   ?
_struct_ncs_oper.matrix[1][2]   ?
_struct_ncs_oper.matrix[1][3]   ?
_struct_ncs_oper.matrix[2][1]   ?
_struct_ncs_oper.matrix[2][2]   ?
_struct_ncs_oper.matrix[2][3]   ?
_struct_ncs_oper.matrix[3][1]   ?
_struct_ncs_oper.matrix[3][2]   ?
_struct_ncs_oper.matrix[3][3]   ?
_struct_ncs_oper.vector[1]      ?
_struct_ncs_oper.vector[2]      ?
_struct_ncs_oper.vector[3]      ?
#
_pdbx_post_process_details.entry_id      1A3I
_pdbx_post_process_details.text          ?
_pdbx_post_process_details.seq_details   ?
#
_pdbx_post_process_status.entry_id     1A3I
_pdbx_post_process_status.cycle_id     ?
_pdbx_post_process_status.date_begin   ?
_pdbx_post_process_status.date_end     ?
_pdbx_post_process_status.details      ?
_pdbx_post_process_status.annotator    ?
#
_refine_hist.pdbx_refine_id                   'X-RAY DIFFRACTION'
_refine_hist.cycle_id                         LAST
_refine_hist.pdbx_number_atoms_protein        0
_refine_hist.pdbx_number_atoms_nucleic_acid   0
_refine_hist.pdbx_number_atoms_ligand         5
_refine_hist.number_atoms_solvent             0
_refine_hist.number_atoms_total               5
_refine_hist.d_res_high                       .
_refine_hist.d_res_low                        .
#
_em_2d_projection_selection.entry_id        1A3I
_em_2d_projection_selection.software_name   ?
_em_2d_projection_selection.num_particles   ?
#
_em_3d_fitting.id                ?
_em_3d_fitting.entry_id          1A3I
_em_3d_fitting.ref_space         ?
_em_3d_fitting.ref_protocol      ?
_em_3d_fitting.target_criteria   ?
_em_3d_fitting.overall_b_value   ?
_em_3d_fitting.method            ?
#
_em_3d_reconstruction.entry_id                    1A3I
_em_3d_reconstruction.id                          ?
_em_3d_reconstruction.method                      ?
_em_3d_reconstruction.nominal_pixel_size          ?
_em_3d_reconstruction.actual_pixel_size           ?
_em_3d_reconstruction.resolution                  ?
_em_3d_reconstruction.ctf_correction_method       ?
_em_3d_reconstruction.magnification_calibration   ?
_em_3d_reconstruction.details                     ?
#
_em_assembly.id                  ?
_em_assembly.entry_id            1A3I
_em_assembly.aggregation_state   ?
_em_assembly.name                ?
_em_assembly.details             ?
#
_em_sample_preparation.entry_id               1A3I
_em_sample_preparation.id                     ?
_em_sample_preparation.buffer_id              ?
_em_sample_preparation.support_id             ?
_em_sample_preparation.sample_concentration   ?
_em_sample_preparation.ph                     ?
#
_em_vitrification.entry_id   1A3I
_em_vitrification.id         ?
_em_vitrification.details    ?
#
_em_experiment.entry_id                1A3I
_em_experiment.reconstruction_method   ?
_em_experiment.specimen_type           ?
#
_em_single_particle_entity.entry_id        1A3I
_em_single_particle_entity.symmetry_type   ?
#
_entry.id   1A3I
#
loop_
_atom_type.symbol
C
N
O
#
loop_
_pdbx_unobs_or_zero_occ_atoms.id
_pdbx_unobs_or_zero_occ_atoms.polymer_flag
_pdbx_unobs_or_zero_occ_atoms.occupancy_flag
_pdbx_unobs_or_zero_occ_atoms.PDB_model_num
_pdbx_unobs_or_zero_occ_atoms.auth_asym_id
_pdbx_unobs_or_zero_occ_atoms.auth_comp_id
_pdbx_unobs_or_zero_occ_atoms.auth_seq_id
_pdbx_unobs_or_zero_occ_atoms.PDB_ins_code
_pdbx_unobs_or_zero_occ_atoms.auth_atom_id
_pdbx_unobs_or_zero_occ_atoms.label_alt_id
1 N 1 1 B PRO 1 ? CG  ?
2 N 1 1 B PRO 1 ? CD  ?
3 N 1 1 B PRO 1 ? OXT ?
#
_pdbx_entity_nonpoly.entity_id   1
_pdbx_entity_nonpoly.name        PROLINE
_pdbx_entity_nonpoly.comp_id     PRO
#
loop_
_atom_site.group_PDB
_atom_site.id
_atom_site.type_symbol
_atom_site.label_atom_id
_atom_site.label_alt_id
_atom_site.label_comp_id
_atom_site.label_asym_id
_atom_site.label_entity_id
_atom_site.label_seq_id
_atom_site.pdbx_label_seq_num
_atom_site.pdbx_PDB_ins_code
_atom_site.Cartn_x
_atom_site.Cartn_y
_atom_site.Cartn_z
_atom_site.occupancy
_atom_site.B_iso_or_equiv
_atom_site.Cartn_x_esd
_atom_site.Cartn_y_esd
_atom_site.Cartn_z_esd
_atom_site.occupancy_esd
_atom_site.B_iso_or_equiv_esd
_atom_site.footnote_id
_atom_site.pdbx_formal_charge
_atom_site.auth_seq_id
_atom_site.auth_comp_id
_atom_site.auth_asym_id
_atom_site.auth_atom_id
_atom_site.pdbx_auth_seq_id
_atom_site.pdbx_auth_comp_id
_atom_site.pdbx_auth_asym_id
_atom_site.pdbx_auth_atom_name
_atom_site.pdbx_PDB_model_num
HETATM 1 N N  . PRO B 1 . 1 ? 8.316 21.206 21.530 1.00 17.44 ? ? ? ? ? ? ? 1 PRO B N  1 PRO B N  1
HETATM 2 C CA . PRO B 1 . 1 ? 7.608 20.729 20.336 1.00 17.44 ? ? ? ? ? ? ? 1 PRO B CA 1 PRO B CA 1
HETATM 3 C C  . PRO B 1 . 1 ? 8.487 20.707 19.092 1.00 17.44 ? ? ? ? ? ? ? 1 PRO B C  1 PRO B C  1
HETATM 4 O O  . PRO B 1 . 1 ? 9.466 21.457 19.005 1.00 17.44 ? ? ? ? ? ? ? 1 PRO B O  1 PRO B O  1
HETATM 5 C CB . PRO B 1 . 1 ? 6.460 21.723 20.211 1.00 22.26 ? ? ? ? ? ? ? 1 PRO B CB 1 PRO B CB 1
#"""
        f = NamedTemporaryFile(delete=False, suffix=".cif")
        f.write(CIFString.encode('utf8'))
        f.close()
        cls.CIFFileName = f.name

    def testGetTransformMatrix(self):
        # create and load atom struct
        from .hexonAtomStruct import saveFiles
        h1Fn, h2Fn = saveFiles()
        aSH = emconv.AtomicStructHandler(h1Fn)
        # should get identity matrix
        matrix, error = aSH.getTransformMatrix(h1Fn, 100, 300)
        self.assertTrue(matrix.shape[0] == matrix.shape[1] and\
               np.allclose(matrix, np.eye(matrix.shape[0])))
        data = np.array([[1, 2], [3, 4]])

        goldMatrix = np.array(
            [[ 9.93196087e-01,  2.55599161e-02, -1.13614363e-01, 1.75187864e+00],
             [-2.25492853e-02,  9.99361764e-01,  2.77054840e-02, 4.47446401e+01],
             [ 1.14250000e-01, -2.49550556e-02,  9.93138552e-01, 3.95671369e+01],
             [ 0.00000000e+00,  0.00000000e+00,  0.00000000e+00, 1.00000000e+00]])
        goldError = 0.8032027904183334

        matrix, error = aSH.getTransformMatrix(h2Fn, -1, -1)
        self.assertTrue( matrix.shape[0] == matrix.shape[1] and\
                        np.allclose(matrix, goldMatrix))
        self.assertAlmostEqual(error, goldError)
        # DEBUG, next three lines save the transformed
        # atom struct to file
        # aSH2 = emconv.AtomicStructHandler(h2Fn)
        # aSH2.transform(matrix)
        # aSH2.write("/tmp/kk.cif")

    def testGetBoundingBox(self):
        # create and load atom struct
        from .hexonAtomStruct import saveFiles
        h1Fn, h2Fn = saveFiles()
        aSH = emconv.AtomicStructHandler(h1Fn)
        [[x1, y1, z1],[x2, y2, z2]] = aSH.getBoundingBox()
        self.assertAlmostEqual(x1, 205.438 - 3, places=2)
        self.assertAlmostEqual(y1, -92.590 - 3, places=2)
        self.assertAlmostEqual(z1, 240.313 - 3, places=2)
        self.assertAlmostEqual(x2, 328.960 + 3, places=2)
        self.assertAlmostEqual(y2,   2.663 + 3, places=2)
        self.assertAlmostEqual(z2, 352.586 + 3, places=2)

    def testFunctionAddAttribute(self):
        from .hexonAtomStruct import saveFiles
        import pwem.convert.atom_struct as convAS
        h1Fn, h2Fn = saveFiles()
        outFileName = '/tmp/scipionAttributes.cif'
        attrName = 'hidrophobicity'

        attrDic = {':asp':	3.49, ':glu': 2.68, ':asn':	2.05, ':gln': 2.36, ':lys':	2.71,
                	 ':arg':	2.58, ':his':	2.06, ':gly':	0.74, ':pro':	2.23, ':ser':	0.84,
                   ':thr':	0.52, ':cys':	-0.13, ':met':	-0.10, ':mse':	-0.10, ':ala':	0.11,
                   ':val':	-0.31, ':ile':	-0.60, ':leu':	-0.55, ':phe':	-0.32, ':trp':	0.30,
                	 ':tyr':	0.68}

        aSH = emconv.AtomicStructHandler()
        cifDic = aSH.readLowLevel(h1Fn)
        cifDic = convAS.addScipionAttribute(cifDic, attrDic, attrName, 'residues')
        aSH._writeLowLevel(outFileName, cifDic)

        cifDicOut = aSH.readLowLevel(outFileName)
        self.assertEqual(cifDicOut[convAS.NAME][0], attrName)
        self.assertEqual(cifDicOut[convAS.RECIP][0], 'residues')
        self.assertEqual(cifDicOut[convAS.SPEC][0], ':asp')
        self.assertEqual(cifDicOut[convAS.VALUE][0], '3.49')




#!/usr/bin/env python
# ***************************************************************************
# * Authors:     Roberto Marabini (roberto@cnb.csic.es)
# *
# * Unidad de Bioinformatica of Centro Nacional de Biotecnologia, CSIC
# *
# * This program is free software; you can redistribute it and/or modify
# * it under the terms of the GNU General Public License as published by
# * the Free Software Foundation; either version 3 of the License, or
# * (at your option) any later version.
# *
# * This program is distributed in the hope that it will be useful,
# * but WITHOUT ANY WARRANTY; without even the implied warranty of
# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# * GNU General Public License for more details.
# *
# * You should have received a copy of the GNU General Public License
# * along with this program; if not, write to the Free Software
# * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
# * 02111-1307  USA
# *
# *  All comments concerning this program package may be sent to the
# *  e-mail address 'scipion@cnb.csic.es'
# *
# **************************************************************************


import unittest
import numpy as np

import pyworkflow.tests as pwtests
import pwem.objects as emobj
from pwem.protocols import ProtImportParticles, ProtSetEditor
from pwem.objects.data import Particle, SetOfParticles, Acquisition, CTFModel
from pwem.constants import ALIGN_PROJ

# projection matrices
mList = [
    [[1., 0., 0., 0.],  # a1
     [0., 1., 0., 0.],
     [0., 0., 1., 0.],
     [0., 0., 0., 1.]],
    [[0.04341204, -0.82959837, 0.5566704, 7.42774284],  # -50, -40,-30
     [0.90961589, 0.26325835, 0.3213938, -20.82490128],
     [-0.41317591, 0.49240388, 0.76604444, 3.33947946],
     [0., 0., 0., 1.]],
    [[0.04341204, 0.82959837, 0.5566704, -7.42774284],  # a3
     [0.90961589, -0.26325835, 0.3213938, 20.82490128],
     [0.41317591, 0.49240388, -0.76604444, 3.33947946],
     [0., 0., 0., 1.]]
]

# operate rotate vector
tOut = [
    [[ 0.000000e-00,  0.000000e+00,  1.000000e+00,  0.000000e+00],
     [ 0.000000e+00,  1.000000e+00,  0.000000e+00,  0.000000e+00],
     [-1.000000e+00,  0.000000e+00,  0.000000e-00,  0.000000e+00],
     [ 0.000000e+00,  0.000000e+00,  0.000000e+00,  1.000000e+00]],
    [[ -0.41317591,   0.49240388,   0.76604444,   3.33947946],
     [  0.90961589,   0.26325835,   0.3213938,  -20.82490128],
     [ -0.04341204,   0.82959837,  -0.5566704,   -7.42774284],
     [  0.,           0.,           0.,           1.        ]],
    [[ 0.41317591,  0.49240388, -0.76604444,  3.33947946],
     [ 0.90961589, -0.26325835,  0.3213938,  20.82490128],
     [-0.04341204, -0.82959837, -0.5566704,   7.42774284],
     [ 0.,          0.,          0.,          1.        ]]
]

# operate rotate ico symmetry
tIcoOut = [
    [[0.,  1.,  0.,  0.],
     [-1., 0.,  0.,  0.],
     [0.,  0.,  1.,  0.],
     [0., 0., 0., 1.]],
    [[0.90961589,   0.26325835,   0.3213938, -20.82490128],
     [-0.04341204,   0.82959837, -0.5566704, -7.42774284],
     [-0.41317591,   0.49240388,  0.76604444, 3.33947946],
     [0., 0., 0., 1.]],
    [[0.90961589,  -0.26325835,  0.3213938,  20.82490128],
     [-0.04341204, -0.82959837, -0.5566704,   7.42774284],
     [0.41317591,   0.49240388, -0.76604444,  3.33947946],
     [0., 0., 0., 1.]]
]

defocusList = [15000., 20000., 25000.]
defocusAngle = [0., 10., 20.]
projSize = 128
samplingRate = 1.5


class TestSets(pwtests.BaseTest):
    """Run different tests related to the editor set protocol."""
    @classmethod
    def setUpClass(cls):
        pwtests.setupTestProject(cls)

    def _createSetOfParticles(
        self,
        setPartSqliteName, partFn,
        doCtf=False):
        # create a set of particles

        self.partSet = SetOfParticles(filename=setPartSqliteName)
        self.partSet.setAlignment(ALIGN_PROJ)
        self.partSet.setAcquisition(
            Acquisition(voltage=300,
                        sphericalAberration=2,
                        amplitudeContrast=0.1,
                        magnification=60000)
                        )

        self.partSet.setSamplingRate(samplingRate)
        if doCtf:
            self.partSet.setHasCTF(True)
        aList = [np.array(m) for m in mList]

        for i, a in enumerate(aList):
            p = Particle()
            if doCtf:
                defocusU = defocusList[i]
                defocusV = defocusList[i]
                ctf = CTFModel(defocusU=defocusU,
                               defocusV=defocusV,
                               defocusAngle=defocusAngle[i])
                ctf.standardize()
                p.setCTF(ctf)

            p.setLocation(i + 1, partFn)
            p.setTransform(emobj.Transform(a))
            self.partSet.append(p)
        self.partSet.write()

    def importData(self, baseFn, objLabel, protType, importFrom):
        prot = self.newProtocol(protType,
                                objLabel=objLabel,
                                filesPath=baseFn,
                                maskPath=baseFn,
                                sqliteFile=baseFn,
                                haveDataBeenPhaseFlipped=False,
                                magnification=10000,
                                samplingRate=samplingRate,
                                importFrom=importFrom
                                )
        self.launchProtocol(prot)
        return prot

    def testOperation(self):
        """Make a trivial operation. Increment defocusU  by one"""
        setPartSqliteName = self.proj.getTmpPath("particles_operate.sqlite")
        setPartName = self.proj.getTmpPath('particles.stk')

        self._createSetOfParticles(setPartSqliteName, setPartName,
                                   doCtf=True)
        protImportProj = self.importData(
            setPartSqliteName,
            "import projection\n operation",
            ProtImportParticles,
            ProtImportParticles.IMPORT_FROM_SCIPION)

        # launch operate set protocol
        protSetEditor = self.newProtocol(ProtSetEditor,
                                         objLabel="operate")
        protSetEditor.inputSet.set(protImportProj)
        protSetEditor.inputSet.setExtended("outputParticles")
        protSetEditor.formula.set(
            'item._ctfModel._defocusU.set('
            'item._ctfModel._defocusU.get() + 1. )')
        self.launchProtocol(protSetEditor)
        for item1, item2 in zip(protSetEditor.outputParticles,
                                protImportProj.outputParticles):
            self.assertAlmostEqual(item1._ctfModel._defocusU.get(),
                                   item2._ctfModel._defocusU.get() + 1)


if __name__ == '__main__':
    unittest.main()



# **************************************************************************
# *
# * Authors:     J.M. De la Rosa Trevin (jmdelarosa@cnb.csic.es)
# *              Laura del Cano         (ldelcano@cnb.csic.es)
# *
# * Unidad de  Bioinformatica of Centro Nacional de Biotecnologia , CSIC
# *
# * This program is free software; you can redistribute it and/or modify
# * it under the terms of the GNU General Public License as published by
# * the Free Software Foundation; either version 3 of the License, or
# * (at your option) any later version.
# *
# * This program is distributed in the hope that it will be useful,
# * but WITHOUT ANY WARRANTY; without even the implied warranty of
# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# * GNU General Public License for more details.
# *
# * You should have received a copy of the GNU General Public License
# * along with this program; if not, write to the Free Software
# * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
# * 02111-1307  USA
# *
# *  All comments concerning this program package may be sent to the
# *  e-mail address 'scipion@cnb.csic.es'
# *
# **************************************************************************
"""
This modules contains data classes related to Random Conical Tilt workflow.
"""

# NOTE: Some of this importS are needed by the mapper,
# not directly in the code

from .data import *


class TiltPair(EMObject):
    def __init__(self, untilted=None, tilted=None, **kwargs):
        EMObject.__init__(self, **kwargs)
        self._untilted = untilted
        self._tilted = tilted
        
    def getUntilted(self):
        return self._untilted
    
    def getTilted(self):
        return self._tilted
    
    def setUntilted(self, untilted):
        self._untilted = untilted
        
    def setTilted(self, tilted):
        self._tilted = tilted        
        
        
class TiltPairSet(EMSet):
    """ Base class for Tilt pairs sets. """
    ITEM_TYPE = None
    
    def __init__(self, **kwargs):
        EMSet.__init__(self, **kwargs)
        self._tilted = None
        self._untilted = None
        
    def getUntilted(self):
        return self._untilted
    
    def getTilted(self):
        return self._tilted
    
    def setUntilted(self, untilted):
        self._untilted = untilted
        
    def setTilted(self, tilted):
        self._tilted = tilted
        
    def getFiles(self):
        filePaths = EMSet.getFiles(self)
        filePaths.update(self.getTilted().getFiles())
        filePaths.update(self.getUntilted().getFiles())
        
        return filePaths 
    
    def _loadClassesDict(self):
        return globals()

    def close(self):
        if self._tilted is not None:
            self._tilted.close()
        if self._untilted is not None:
            self._untilted.close()
        EMSet.close(self)

    def write(self, properties=True):
        if self._tilted is not None:
            self._tilted.write(properties=properties)
        if self._untilted is not None:
            self._untilted.write(properties=properties)
        EMSet.write(self, properties=properties)


class MicrographsTiltPair(TiltPairSet):
    """Represents a Micrographs Tilt Pair"""
    ITEM_TYPE = TiltPair

    
class CoordinatesTiltPair(TiltPairSet):
    """Represents a Coordinates Tilt Pair"""
    ITEM_TYPE = TiltPair
    
    def __init__(self, **kwargs):
        TiltPairSet.__init__(self, **kwargs)
        self._angles = None
        self._micsPair = Pointer()
        
    def getAngles(self):
        return self._angles
    
    def getBoxSize(self):
        return self.getUntilted().getBoxSize()
    
    def getMicsPair(self):
        return self._micsPair.get()
    
    def setAngles(self, setAngles):
        self._angles = setAngles
        
    def setMicsPair(self, micsPair):
        self._micsPair.set(micsPair)
        
    def getFiles(self):
        filePaths = TiltPairSet.getFiles(self)
        filePaths.update(self.getAngles().getFiles())
        filePaths.update(self.getMicsPair().getFiles())
        
        return filePaths

    def _loadClassesDict(self):
        return globals()

    def close(self):
        TiltPairSet.close(self)
        if self.getAngles():
            self.getAngles().close()

         
class Angles(EMObject):
    """Represents a triplet of angles"""

    def __init__(self, **args):
        EMObject.__init__(self, **args)
        self._angleY = Float()    
        self._angleY2 = Float()
        self._angleTilt = Float()
        
    def setAngles(self, angleY, angleY2, angleTilt):
        self._angleY.set(angleY)
        self._angleY2.set(angleY2)
        self._angleTilt.set(angleTilt)
        
    def getAngles(self):
        return self._angleY.get(), self._angleY2.get(), self._angleTilt.get()
    
    
class SetOfAngles(EMSet):
    """ Represents a set of Images """
    ITEM_TYPE = Angles
    
    def _loadClassesDict(self):
        return globals()  
    
    
class ParticlesTiltPair(TiltPairSet):
    """Represents a Particles Tilt Pair"""
    ITEM_TYPE = TiltPair
    
    def __init__(self, **kwargs):
        TiltPairSet.__init__(self, **kwargs)
        self._coordsPair = Pointer()
        
    def getCoordsPair(self):
        return self._coordsPair.get()
        
    def setCoordsPair(self, coordsPair):
        self._coordsPair.set(coordsPair)
                
    def getFiles(self):
        filePaths = TiltPairSet.getFiles(self)
        filePaths.update(self.getCoordsPair().getFiles())
        
        return filePaths
